unit Monsters;
// 

interface

uses npUtils, windows, eXgine, GlobalConst;

const mtrMinNumLevel = 5; //.-    +  
      mtrMaxNumLevel = 15;//.-    +  
      mtrMaxType     = 10;//.-  
      poisonDmg      = 3; //  
      poisonTime     = 9; //  

type TMonsterSpecs = (mtrPoison,mtrNone);

type TMonster = record
       Name:      String[20];   //
       ID:        integer;      // 
       x,y:       Integer;      //
       HP, MaxHP: Integer;      //   
       XP:        Integer;      //  
       Level:     Integer;      // 
       Ad1, Ad2:  Integer;      //1,2
       Dd1:       Integer;      //
       ViewZone:  Integer;      // 
       Specs:     TMonsterSpecs;//.
       Paralize:  Integer;      //
     end;

//  
var BaseMonsters: array[1..mtrMaxType] of TMonster;

var MonstersMap: array of TMonster;
    texMonster:  array[1..mtrMaxType] of cardinal;

procedure MonsterCreate(id,x,y: integer);        // 
procedure MonsterTexInit;                        //  
procedure MonsterGenerateLevel;                  //   
procedure NewMonsterCreate;                      //             -
procedure MonsterDraw;                           // 
procedure MonsterDead(mtrNumDead: integer);      // 
procedure FreeTileForMonster(var x,y: integer);  //    
procedure MonsterIncHP(mtrNum, deltaHp: integer);//  
procedure AllMonsterMove;                        //  
procedure MonsterMove(var tmpMtr: TMonster);     //  
procedure MonsterAttack(tmpMtr: TMonster);       // 
procedure PoisonHero;                            //   

implementation

uses Map, Hero, Main, GUI;

//------------------------------------------------------------------------------
procedure MonsterCreate(id,x,y: integer);
var mtrNum: integer;
begin
  SetLength(MonstersMap, Length(MonstersMap)+1);
  mtrNum:= Length(MonstersMap)-1;
  MonstersMap[mtrNum]:= BaseMonsters[id];
  MonstersMap[mtrNum].HP:= BaseMonsters[id].MaxHP;
  MonstersMap[mtrNum].x:= x;
  MonstersMap[mtrNum].y:= y;
end;

//------------------------------------------------------------------------------
procedure MonsterTexInit;
var mtrNum: integer;
begin
  for mtrNum:= 1 to mtrMaxType do
    texMonster[mtrNum]:= tex.Load(PChar('data/monster'+inttostr(mtrNum)+'.tga'),false,false);
end;

//------------------------------------------------------------------------------
procedure MonsterDraw;
var mtrNum: integer;
begin
  for mtrNum:= 0 to length(MonstersMap)-1 do
  begin
    if (CurMap.MapX <= MonstersMap[mtrNum].X)
    and (MonstersMap[mtrNum].X <= CurMap.MapX+viewScreen-1)
    and (CurMap.MapY <= MonstersMap[mtrNum].Y)
    and (MonstersMap[mtrNum].Y <= CurMap.MapY+viewScreen-1)
    and (CurMap.MapCell[MonstersMap[mtrNum].X,MonstersMap[mtrNum].Y].visible = true) then
      DrawSprite2D_simple(texMonster[MonstersMap[mtrNum].ID],
      startDrawX+(MonstersMap[mtrNum].X-CurMap.MapX)*TileW,
      startDrawY+(MonstersMap[mtrNum].Y-CurMap.MapY)*tileH,TileW,TileH);
  end;
end;

//------------------------------------------------------------------------------
procedure MonsterDead(mtrNumDead: integer);
var mtrNum: integer;
begin
  inc(Player.Exp,MonstersMap[mtrNumDead].XP);
  OutTextAdd(MonstersMap[mtrNumDead].Name+' .'+'  '
             +inttostr(MonstersMap[mtrNumDead].XP));
  for mtrNum:= mtrNumDead to length(MonstersMap)-2 do
    MonstersMap[mtrNum]:= MonstersMap[mtrNum+1];
  SetLength(MonstersMap,length(MonstersMap)-1);
end;

//------------------------------------------------------------------------------
procedure MonsterGenerateLevel;
var mtrRndNum:  integer;
    mtrNum:     integer;
    mtrX, mtrY: integer;
    mtrId:      integer;
begin
  SetLength(MonstersMap,0);
  mtrRndNum:= Random(mtrMaxNumLevel-mtrMinNumLevel)+
              mtrMinNumLevel+Round(CurMap.Level/5);
  for mtrNum:= 1 to mtrRndNum do
  begin
    mtrId:= Random(mtrMaxType)+1;
    while ((BaseMonsters[mtrId].Level*5 > CurMap.Level) and
          (BaseMonsters[mtrId].Level*12 > CurMap.Level)) or
          ((CurMap.Level >=10) and (BaseMonsters[mtrId].Level= 0)) do
    begin
      mtrId:= Random(mtrMaxType)+1;
    end;
    FreeTileForMonster(mtrX,mtrY);
    MonsterCreate(mtrId,mtrX,mtrY);
  end;
end;

//------------------------------------------------------------------------------
procedure NewMonsterCreate;
var mtrX, mtrY: integer;
    mtrId:      integer;
begin
  if Length(MonstersMap) <= 3 then
  begin
    mtrId:= Random(mtrMaxType)+1;
    while ((BaseMonsters[mtrId].Level*5 > CurMap.Level) and
          (BaseMonsters[mtrId].Level*12 > CurMap.Level)) or
          ((CurMap.Level >=10) and (BaseMonsters[mtrId].Level= 0)) do
    begin
      mtrId:= Random(mtrMaxType)+1;
    end;
    FreeTileForMonster(mtrX,mtrY);
    MonsterCreate(mtrId,mtrX,mtrY);
  end;
end;

//------------------------------------------------------------------------------
procedure FreeTileForMonster(var x,y: integer);
var mtrNum:      integer;
    mtrX, mtrY:  integer;
    mtrFreeTile: boolean;
begin
  mtrFreeTile:= false;
  while not mtrFreeTile do
  begin
    mtrFreeTile:= true;
    FreeMapPoint(mtrX,mtrY,flagAllFree);
    if (Player.FCoordX = mtrX) and
       (Player.FCoordY = mtrY) then
       mtrFreeTile:= false;
    for mtrNum:= 0 to Length(MonstersMap)-1 do
    begin
      if (MonstersMap[mtrNum].x = mtrX) and
         (MonstersMap[mtrNum].y = mtrY) then
         mtrFreeTile:= false;
    end;
  end;
  x:= mtrX;
  y:= mtrY;
end;

//------------------------------------------------------------------------------
procedure MonsterIncHP(mtrNum, deltaHp: integer);
begin
  inc(MonstersMap[mtrNum].HP,deltaHp);
  BloodInit(MonstersMap[mtrNum].x,MonstersMap[mtrNum].y,true);
  if MonstersMap[mtrNum].HP <= 0 then
    MonsterDead(mtrNum)
  else if MonstersMap[mtrNum].HP > MonstersMap[mtrNum].MaxHP then
    MonstersMap[mtrNum].HP:= MonstersMap[mtrNum].MaxHP;
end;

//------------------------------------------------------------------------------
procedure AllMonsterMove;
var mtrNum:  integer;
    rndMove: integer;
begin
  for mtrNum:= 0 to Length(MonstersMap)-1 do
  begin
    if MonstersMap[mtrNum].Paralize > 0 then
    begin
      // 
      MonstersMap[mtrNum].Paralize:= MonstersMap[mtrNum].Paralize-1;
      break;
    end;
    if ((Player.FCoordY = MonstersMap[mtrNum].y) and
       ((Player.FCoordX = MonstersMap[mtrNum].x-1) or
       (Player.FCoordX = MonstersMap[mtrNum].x+1))) or
       ((Player.FCoordX = MonstersMap[mtrNum].x) and
       ((Player.FCoordY = MonstersMap[mtrNum].y-1) or
       (Player.FCoordY = MonstersMap[mtrNum].y+1))) then
    begin
      //   ,  
      MonsterAttack(MonstersMap[mtrNum]);
    end
    else
    begin
      rndMove:= Random(100)+1;
      //        
      if rndMove > 5 then
        MonsterMove(MonstersMap[mtrNum]);
    end;
  end;
end;

//------------------------------------------------------------------------------
procedure MonsterMove(var tmpMtr: TMonster);
var rndVector:   integer;
    flagMove:    boolean;
    dx,dy:       integer;
function FreePathMonster(deltaX,deltaY: integer): boolean;//   
var mtrNum:      integer;
    mtrFreeTile: boolean;
begin
  FreePathMonster:= false;
  mtrFreeTile:= true;
  for mtrNum:= 0 to Length(MonstersMap)-1 do
  begin
    if (MonstersMap[mtrNum].x = tmpMtr.x+deltaX) and
       (MonstersMap[mtrNum].y = tmpMtr.y+deltaY) then
       mtrFreeTile:= false;
  end;
  if (FreeTile(CurMap.MapCell[tmpMtr.x+deltaX,tmpMtr.y+deltaY].tile,flagAllFree) = true)
    and (mtrFreeTile = true) then
    FreePathMonster:= true;
end;
begin
  if (Player.FCoordX-tmpMtr.ViewZone < tmpMtr.x) and
     (Player.FCoordX+tmpMtr.ViewZone > tmpMtr.x) and
     (Player.FCoordY-tmpMtr.ViewZone < tmpMtr.y) and
     (Player.FCoordY+tmpMtr.ViewZone > tmpMtr.y) then
  begin
    //    
    //     X  Y     
    if Player.FCoordX > tmpMtr.x then
      dx := 1
    else if Player.FCoordX < tmpMtr.x then
      dx := -1
    else
      dx := 0;
    if Player.FCoordY > tmpMtr.y then
      dy := 1
    else if Player.FCoordY < tmpMtr.y then
      dy := -1
    else
      dy := 0;
   if dx <> 0 then
     if not FreePathMonster(dx,0) then
       dx := 0;
   if dy <> 0 then
     if not FreePathMonster(0,dy) then
       dy := 0;
   if (dx = 0) and (dy = 0) then Exit;
   if dx = 0 then
     tmpMtr.y:= tmpMtr.y+dy
   else if dy = 0 then
     tmpMtr.x:= tmpMtr.x+dx
   else if random(2) = 0 then
     tmpMtr.y:= tmpMtr.y+dy
   else
     tmpMtr.x:= tmpMtr.x+dx;
  end
  else
  begin
    // ,      
    flagMove:= false;
    if (FreePathMonster(-1,0) = false) and (FreePathMonster(1,0) = false) and
       (FreePathMonster(0,-1) = false) and (FreePathMonster(0,1) = false) then
       exit;
    while not flagMove do
    begin
      rndVector:= Random(4);
      if (rndVector = vectorLeft) and (FreePathMonster(-1,0) = true) then
      begin
        tmpMtr.x:= tmpMtr.x-1;
        flagMove:= true;
      end;
      if (rndVector = vectorTop) and (FreePathMonster(0,-1) = true) then
      begin
        tmpMtr.y:= tmpMtr.y-1;
        flagMove:= true;
      end;
      if (rndVector = vectorRight) and (FreePathMonster(1,0) = true) then
      begin
        tmpMtr.x:= tmpMtr.x+1;
        flagMove:= true;
      end;
      if (rndVector = vectorDown) and (FreePathMonster(0,1) = true) then
      begin
        tmpMtr.y:= tmpMtr.y+1;
        flagMove:= true;
      end;
    end;
  end;
end;

//------------------------------------------------------------------------------
procedure MonsterAttack(tmpMtr: TMonster);
var mtrDamage: integer;
    mtrMissed: integer;
begin
  mtrMissed:= Random(100)+1+(HeroCoeffFatigue*10);
  if mtrMissed > 50 then
  begin
    mtrDamage:= RollDice(tmpMtr.Ad1,tmpMtr.Ad2)-Player.armor;
    if  mtrDamage <= 0 then mtrDamage:= 1;
    OutTextAdd(tmpMtr.Name+'   '+inttostr(mtrDamage));
    if tmpMtr.Specs = mtrPoison then
      PoisonHero;
    HeroIncHP(-mtrDamage);
  end
  else
  OutTextAdd(' ');
end;

//------------------------------------------------------------------------------
procedure PoisonHero;
var rndPsn: integer;
begin
  rndPsn:= Random(100)+1;
  if rndPsn > 70 then
  begin
    Player.Poison:= poisonTime;
    OutTextAdd(' !');
  end;
end;

end.
