unit Billboards;

interface
uses
  OpenGL, Windows, GameTypes, GLTYpes, u_opengl;

type
  TBillboard = class (TGameObject)
  public
    Size : Single;
    tId : Integer;
    procedure Render; override;
    constructor Create(Texture : String; Sz : Single); overload;
    constructor Create(TexID : Integer; Sz : Single); overload;
  end;

  TSmokeBBD = class (TBillboard)
  public
    procedure Move; override;
  end;

implementation
uses
  TextureManager;

{ TBillboard }

constructor TBillboard.Create(Texture: String; Sz: Single);
begin
  tId := TextureMngr.LoadTexture(Texture, True);
  Size := Sz;
end;

constructor TBillboard.Create(TexID: Integer; Sz : Single);
begin
  tId := TexID;
  Size := Sz;
  Blend := True;
end;

procedure TBillboard.Render;
var Cross1, Cross2,
    A, B, C, D: TGLVector;
    Matrix: array [0..15] of Single;
begin
  glPushMatrix;
 glGetFloatv(GL_MODELVIEW_MATRIX, @Matrix);

 Cross1 := CreateVector(Matrix[0], Matrix[4], Matrix[8]);
 Cross1 := MultyplyVectorScalar(Cross1, Size);

 Cross2 := CreateVector(Matrix[1], Matrix[5], Matrix[9]);
 Cross2 := MultyplyVectorScalar(Cross2, Size);

 //glDepthMask(False);
 glEnable(GL_BLEND);
 glBlendFunc(GL_ONE, GL_ONE);

 A := AddVectors(Pos, CreateVector(- Cross1.X - Cross2.X, - Cross1.Y - Cross2.Y, - Cross1.Z - Cross2.Z));
 B := AddVectors(Pos, CreateVector(Cross1.X - Cross2.X, Cross1.Y - Cross2.Y, Cross1.Z - Cross2.Z));
 C := AddVectors(Pos, CreateVector(Cross1.X + Cross2.X, Cross1.Y + Cross2.Y, Cross1.Z + Cross2.Z));
 D := AddVectors(Pos, CreateVector(- Cross1.X + Cross2.X, - Cross1.Y + Cross2.Y, - Cross1.Z + Cross2.Z));

 OpenGL_EnableTextures;
 TextureMngr.Bind(tId);
 glColor3f(1, 1, 1);
 glBegin(GL_QUADS);
  glTexCoord2f(0.0, 0.0); glVertex3fv(@A);
  glTexCoord2f(1.0, 0.0); glVertex3fv(@B);
  glTexCoord2f(1.0, 1.0); glVertex3fv(@C);
  glTexCoord2f(0.0, 1.0); glVertex3fv(@D);
 glEnd;
 OpenGL_DisableTextures;

 glDisable(GL_BLEND);
// glDepthMask(True);
 glPopMatrix;
end;

{ TSmokeBBD }

procedure TSmokeBBD.Move;
begin
  if Size < 1 then Size := Size + 0.01 else MustDie := True;
end;

end.
