unit GLTypes;

interface
uses
  OpenGL;

type
  TGLVector = record
    X,Y,Z:GLfloat;
  end;

  TGLFace = record
    v1,v2,v3 : Integer;
  end;

  TGLTexCoord = record
    u,v:Single;
  end;

function AddVectors(v1, v2 : TGLVector):TGLVector;
function CreateVector(x,y,z : GLfloat):TGLVector;
function Trigon2DVector(Angle : GLfloat):TGLVector;
function Trigon3DVector(Va, Ha : GLfloat):TGLVector;
function SubVectors(v1, v2 : TGLVector):TGLVector;
function MultyplyVectorScalar(v : TGLVector; Scalar : GLfloat):TGLVector;
function NormalizeVector(v : TGLVector):TGLVector;
function Distance(Pos1, Pos2 : TGLVector) : Single;

function AngleTo(x0,z0,x1,z1:Single):Single;

implementation

function AngleTo(x0,z0,x1,z1:Single):Single;
var
  a:Single;
begin
  x1 := x1-x0;
  z1 := z1-z0;
  a := 0;
  if(x1=0)then
  begin
    if z1>0 then a:=90;
    if z1<0 then a:=270;
  end else
  begin
    a := (z1/x1);
    a := abs(arctan(a))*(180/pi);
  end;

  
  if (x1<0)and(z1>0)then a:=180-a;
  if (x1<0)and(z1<0)then a:=180+a;
  if (x1>0)and(z1<0)then a:=360-a;

  Result := a;

end;

function Distance;
begin
Result := sqrt(sqr(Pos2.x-Pos1.x)+sqr(Pos2.y-Pos1.y)+sqr(Pos2.z-Pos1.z));
end;

function NormalizeVector(v : TGLVector):TGLVector;
var
  d:Single;
begin
  d:=sqrt(sqr(v.X)+sqr(v.y)+sqr(v.z));
  if d = 0 then d :=1;
  Result.x:=v.x/d;
  Result.y:=v.y/d;
  Result.z:=v.z/d;
end;

function MultyplyVectorScalar(v : TGLVector; Scalar : GLfloat):TGLVector;
begin
  Result.X := v.X * Scalar;
  Result.Y := v.Y * Scalar;
  Result.Z := v.Z * Scalar;
end;

function SubVectors(v1, v2 : TGLVector):TGLVector;
begin
  Result.X := v1.X - v2.X;
  Result.Y := v1.Y - v2.Y;
  Result.Z := v1.Z - v2.Z;
end;

function Trigon3DVector(Va, Ha : GLfloat):TGLVector;
var
  sm : GLfloat;
begin
  sm := Cos(Ha / (180 / pi));
  Result.X := Cos(Va / (180 / pi)) * sm;
  Result.Z := Sin(Va / (180 / pi)) * sm;
  Result.Y := Sin(Ha / (180 / pi));
end;

function Trigon2DVector(Angle : GLfloat):TGLVector;
begin
  Result.X := Cos(Angle / (180 / pi));
  Result.Z := Sin(Angle / (180 / Pi));
  Result.Y := 0;
end;

function CreateVector(x,y,z : GLfloat):TGLVector;
begin
  Result.X := x;
  Result.Y := y;
  Result.Z := z;
end;

function AddVectors(v1, v2 : TGLVector):TGLVector;
begin
  Result.X := v1.X + v2.X;
  Result.Y := v1.Y + v2.Y;
  Result.Z := v1.Z + v2.Z;
end;

end.
