unit HUD;

interface
uses
  OpenGL, Windows, u_OpenGL, GameTypes, GLTypes, u_winapi, TextureManager, Utils;
type
  TFont = class
  private
    texture_id : GLuint;
    CharSize, ClipCharX, ClipCharY, StartChar : Byte;
    tWidth, tHeight : Integer;
  public
    constructor Create(TextureFile : String; nCharSize, nClipCharX, nClipCharY : Byte; FirstChar : Char);
    procedure Writeln(X,Y:Integer; Text : String; r : Byte = 255; g : byte = 255; b:Byte = 255);
  end;

  TSystemFont = class
  private
    FontHandle : GLuint;
  public
    constructor Create(FontName : String; Size : Integer; bold : boolean = false; italic : boolean = false);
    procedure Print(X, Y : Integer; Text : String; r : byte = 255; g : byte = 255; b : Byte = 255);
  end;

  TRadar = class
    procedure Render; virtual; abstract;
    procedure Move; virtual; abstract;
  end;

  THUD = class
  private
    PanelBG,
    Aim, tTarget : GLuint;
    
    Debug,
    Font : TSystemFont;
  public
    Radar : TRadar;
    Info : array[0..1] of String;
    procedure SetOrtho;
    procedure Render;
    procedure Move;
    constructor Create;
  end;

  TBasicRadar = class (TRadar)
    Parent : THUD;
    procedure Render; override;
    procedure Move; override;
  end;

  TRefuelRadar = class (TBasicRadar)
    Cab,
    Conn : Integer;
    Sx, Sy : Single;
    constructor Create;
    procedure Render; override;
    procedure Move; override;
  end;

  TLendRadar = class (TBasicRadar)
    Font : TSystemFont;
    Plan : Integer;
    procedure Render; override;
    procedure Move; override;
    constructor Create;
  end;

const
  RADAR_WIDTH = 320;
  RADAR_HEIGHT = 260;

implementation

uses TheGame;

{ THUD }

constructor THUD.Create;
begin
  PanelBG := TextureMngr.LoadTexture('data\textures\panel.tga', False);
  Aim := TextureMngr.LoadTexture('data\textures\gui\aim_r.tga', False);
  tTarget := TextureMngr.LoadTexture('data\textures\gui\target.tga');
  Debug := TSystemFont.Create('Courier New', 15);
  Font := TSystemFont.Create('Fixedsys', 15);
end;

procedure THUD.Move;
begin
  if Radar <> nil then
    Radar.Move;
end;

procedure THUD.Render;
var
  PanY : Single;
  col : record
    r,g,b:Byte;
  end;
Begin
  SetOrtho;

  glDisable(GL_DEPTH_TEST);

 { Debug.Print(10, 20, 'Player Alt   : ' + FtoA(Game.Player.Alt, 5));
  Debug.Print(10, 40, 'Player Tilt  : ' + FtoA(Game.Player.Tilt, 5));
  Debug.Print(10, 60, 'Player Speed : ' + FtoA(Game.Player.Speed, 5));
  Debug.Print(10, 80, 'Player Fuel : ' + FtoA(Game.Player.Fuel, 5));
  Debug.Print(10, 100, 'Camera.AHor: ' + FtoA(GAme.Camera.AHor));
  Debug.Print(10, 120, 'Camera.AVert: ' + FtoA(GAme.Camera.AVert));
  Debug.Print(10, 140, 'Total objects: ' + ItoA(Length(Game.Objects)));}

  glColor3f(1, 1, 1);
  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  OpenGL_EnableTextures;
  TextureMngr.Bind(PanelBG);
  PanY := -200;
  glBegin(GL_QUADS);
    glTexCoord2f(0, 1); glVertex2f(0, PAnY);
    glTexCoord2f(1, 1); glVertex2f(Screen_Width, PanY);
    glTexCoord2f(1, 0); glVertex2f(Screen_Width, Screen_Height);
    glTexCoord2f(0, 0); glVertex2f(0, Screen_Height);
  glEnd;

  if Game.Player.AimTo <> nil then
  begin
    TextureMngr.Bind(Aim);
    glBegin(GL_QUADS);
      glTexCoord2f(0, 1); glVertex2i(Game.Player.AimTo.scrX-16, Game.Player.AimTo.scrY-16);
      glTexCoord2f(1, 1); glVertex2i(Game.Player.AimTo.scrX+16, Game.Player.AimTo.scrY-16);
      glTexCoord2f(1, 0); glVertex2i(Game.Player.AimTo.scrX+16, Game.Player.AimTo.scrY+16);
      glTexCoord2f(0, 0); glVertex2i(Game.Player.AimTo.scrX-16, Game.Player.AimTo.scrY+16);
    glEnd;

    //Debug.Print(Game.Player.AimTo.scrX, Game.Player.AimTo.scrY-32, Game.Player.AimTo.TypeName, 255, 0, 0);
  end;

  glBlendFunc(GL_ONE, GL_ONE);
  TextureMngr.Bind(tTarget);
  glBegin(GL_QUADS);
    glTexCoord2f(0, 0); glVertex2i(Screen_Width div 2 - 8, Screen_Height div 3 - 8);
    glTexCoord2f(1, 0); glVertex2i(Screen_Width div 2 + 8, Screen_Height div 3 - 8);
    glTexCoord2f(1, 1); glVertex2i(Screen_Width div 2 + 8, Screen_Height div 3 + 8);
    glTexCoord2f(0, 1); glVertex2i(Screen_Width div 2 - 8, Screen_Height div 3 + 8);
  glEnd;

  OpenGL_DisableTextures;
  glDisable(GL_BLEND);

  Font.Print(360, 700, Info[0], 0, 255, 0);
  Font.Print(360, 735, Info[1], 0, 255, 0);


  col.r := 200;
  col.g := 200;
  col.b := 200;

  Font.Print(150, 600, 'SPEED : ' + FtoA(Game.Player.Speed * 1000) + 'km/h', col.r, col.g, col.b);
  Font.Print(150, 620, 'ALT   : ' + FtoA(Game.Player.Alt * 10) + 'm', col.r, col.g, col.b);
  Font.Print(150, 660, 'TILT  : ' + FtoA(Game.Player.Tilt) + '', col.r, col.g, col.b);
  Font.Print(150, 680, 'MISS. : ' + ItoA(Game.Player.RocketCnt) + '/' + ItoA(Game.Player.RocketTTL), col.r, col.g, col.b);
  if Game.Player.Fuel < 60 then
  begin
    col.r := 255;
    col.g := 255;
    col.b := 0;
  end;

  if Game.Player.Fuel < 15 then
  begin
    col.r := 255;
    col.g := 0;
    col.b := 0;
  end;
  Font.Print(150, 640, 'FUEL  : ' + FtoA(Game.Player.Fuel) + '%', col.r, col.g, col.b);
  
  if Radar <> nil then
  begin
    glTranslatef(355, 480, 0);
    Radar.Render;
  end;

  glEnable(GL_DEPTH_TEST);
  
end;

procedure THUD.SetOrtho;
begin
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity;

  gluOrtho2D(0, Screen_Width, Screen_Height, 0);

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity;
end;

{ TFont }

constructor TFont.Create(TextureFile: String; nCharSize, nClipCharX,
  nClipCharY: Byte; FirstChar: Char);
begin
  texture_id := TextureMngr.LoadTexture(TextureFile);
  CharSize := nCharSize;
  ClipCharX := nClipCharX;
  ClipCharY := nClipCharY;
  StartChar := Byte(FirstChar);
  tWidth := TextureMngr.Textures[texture_id].Width;
  tHeight := TextureMngr.Textures[texture_id].Height;
end;

procedure TFont.Writeln(X, Y: Integer; Text: String; r, g, b: byte);
var
  i, texPosX, texPosY:Integer;
  CharTextureWidth, CharTextureHeight : Single;
begin
  glPushMatrix;
  glTranslatef(X, Y, 0);
  glColor4ub(r, g, b, 255);
  TextureMngr.Bind(texture_id);
  glBegin(GL_QUADS);

  CharTextureWidth := 1 / (tWidth div CharSize);
  CharTextureHeight := 1 / (tHeight div CharSize);

  for i := 0 to length(Text)-1 do
  begin
    texPosX := (byte(Text[i+1])-Byte(StartChar)) mod (tWidth div CharSize);
    texPosY := (byte(Text[i+1])-Byte(StartChar)) div (tWidth div CharSize) - 1;

    glTexCoord2f(CharTextureWidth*texPosX,
                 CharTextureHeight*texPosY+CharTextureHeight);

    glVertex2i(i*CharSize, 0);

    glTexCoord2f(CharTextureWidth*texPosX + CharTextureWidth,
                 CharTextureHeight*texPosY+CharTextureHeight);

    glVertex2i(i*CharSize+CharSize, 0);

    glTexCoord2f(CharTextureWidth*texPosX + CharTextureWidth,
                 CharTextureHeight*texPosY);

    glVertex2i(i*CharSize+CharSize, CharSize);

    glTexCoord2f(CharTextureWidth*texPosX,
                 CharTextureHeight*texPosY);

    glVertex2i(i*CharSize, CharSize);
  end;

  glEnd;
  glPopMatrix;
end;

{ TSystemFont }

constructor TSystemFont.Create(FontName: String; Size: Integer; bold,
  italic: boolean);
var
  font : HFONT;
  Weight : Integer;
begin
  if bold then
    Weight := FW_BOLD else
      Weight := FW_NORMAL;
  FontHandle := glGenLists(256);
  font := CreateFont(Size, Size div 2, 0, 0, Weight, DWORD(italic), 0, 0, DEFAULT_CHARSET, OUT_TT_PRECIS, CLIP_DEFAULT_PRECIS, DRAFT_QUALITY, FF_DONTCARE or DEFAULT_PITCH, PChar(FontName));
  SelectObject(DC, font);
  wglUseFontBitmaps(DC, 0, 256, FontHandle);
  DeleteObject(font);
end;

procedure TSystemFont.Print(X, Y: Integer; Text: String; r, g, b: Byte);
begin
  glColor3ub(r, g, b);
  glRasterPos2i(X, Y);
  glListBase(FontHandle);
  glCallLists(length(Text), GL_UNSIGNED_BYTE, @Text[1]);
end;

{ TRefuelRadar }

constructor TRefuelRadar.Create;
begin
  Cab := TextureMngr.LoadTexture('data\textures\radar\refueling_cab.tga');
  Conn := TextureMngr.LoadTexture('data\textures\radar\connector.tga');
end;

procedure TRefuelRadar.Move;
begin
  inherited;
  if Keys[ord('A')] then
    Sx := Sx - 0.5 else
  if Keys[ord('D')] then
    Sx := Sx + 0.5;
end;

procedure TRefuelRadar.Render;
begin
  inherited;
  TextureMngr.Bind(Cab);
  glBegin(GL_QUADS);
    glTexCoord2f(0, 1); glVertex2i(0, 0);
    glTexCoord2f(1, 1); glVertex2i(320, 0);
    glTexCoord2f(1, 0); glVertex2i(320, 260);
    glTexCoord2f(0, 0); glVertex2i(0, 260);
  glEnd;

  TextureMngr.Bind(Conn);
  glBegin(GL_QUADS);
    glTexCoord2f(0, 1); glVertex2f(Sx-80, Sy);
    glTexCoord2f(1, 1); glVertex2f(Sx+80, Sy);
    glTexCoord2f(1, 0); glVertex2f(Sx+80, Sy+140);
    glTexCoord2f(0, 0); glVertex2f(Sx-80, Sy+140);
  glEnd;
end;

{ TBasicRadar }

procedure TBasicRadar.Move;
begin
  inherited;
//
end;

procedure TBasicRadar.Render;
var
  i:Integer;
begin
  glColor3f(0, 0.4, 0);
  glBegin(GL_LINES);
  for i := 0 to 8 do
  begin
    glVertex2i(i*40, 0);
    glVertex2i(i*40, 260);
  end;
  for i := 0 to 8 do
  begin
    glVertex2i(0, i*40);
    glVertex2i(320, i*40);
  end;
  glEnd;

  glTranslatef(160, 130, 0);
  glBegin(GL_TRIANGLES);
    glVertex2i(0, -10);
    glVertex2i(-10, 10);
    glVertex2i(10, 10);
  glEnd;
  glRotatef(-Game.Player.HAngle-180, 0.0, 0.0, 1.0);

  glPointSize(3);  
  glBegin(GL_POINTS);
  for i := 0 to High(Game.Objects) do
  begin
    if Distance(Game.Player.Pos, Game.Objects[i].Pos) < 100 then
    begin
      if Game.Objects[i].TypeName = 'Mig 28' then
      begin
        glColor3f(1, 0, 0);
      end else
      if Game.Objects[i].TypeName = 'Rocket' then
      begin
        glColor3f(0, 1, 0);
      end else
        Continue;

      glVertex3f(Game.Objects[i].Pos.X-Game.Player.Pos.X, Game.Objects[i].Pos.Z - Game.Player.Pos.Z, 0);
    end;
  end;
  glEnd;
  glPointSize(1);
end;

{ TLendRadar }

constructor TLendRadar.Create;
begin
  Plan := TextureMngr.LoadTexture('data\textures\radar\tilt.tga', True); 
end;

procedure TLendRadar.Move;
begin
  inherited;
end;

procedure TLendRadar.Render;
begin
  glColor3f(0, 0.7, 0.0);
  {glBegin(GL_LINES);
    glVertex2i(0, RADAR_HEIGHT div 2);
    glVertex2i(RADAR_WIDTH, RADAR_HEIGHT div 2);
  glEnd;}

  OpenGL_EnableTextures;
  TextureMngr.Bind(Plan);
  glEnable(GL_BLEND);
  glBlendFunc(GL_ONE, GL_ONE);

  glTranslatef(RADAR_WIDTH div 2, RADAR_HEIGHT div 2, 0);
  glRotatef(Game.Player.Tilt, 0.0, 0.0, 1.0);
  glBegin(GL_QUADS);
    glTexCoord2f(1, 1); glVertex2i(-RADAR_WIDTH div 2, -RADAR_WIDTH div 10);
    glTexCoord2f(0, 1); glVertex2i( RADAR_WIDTH div 2, -RADAR_WIDTH div 10);
    glTexCoord2f(0, 0); glVertex2i( RADAR_WIDTH div 2,  RADAR_WIDTH div 10);
    glTexCoord2f(1, 0); glVertex2i(-RADAR_WIDTH div 2,  RADAR_WIDTH div 10);
  glEnd;

  OpenGL_DisableBlend;
  OpenGL_DisableTextures;
end;

end.
