unit MissionStart;

interface
uses
  Windows, OpenGL, HUD, u_winapi, u_opengl, g_global_vars, GameTypes, GLCamera, GLTypes, Projectiles;
  
type
  TMissionBrief = class
  private
    Music1,
    Music2,
    Map : Integer;
    Mode : DWORD;
    Font : TSystemFont;
    Strings : array of String;
    Rockets : array[0..2] of TProjectile;
    elp : Single;
    Cam : TGLCamera;
    SelMiss : Integer;
    procedure SetText(s : String);
    function GetText : String;
  public
    property Text : String read GetText write SetText;
    procedure Render;
    procedure Move;
    procedure KeyDown(Key : Byte);
    constructor Create;
  end;

var
  MissionBrief : TMissionBrief;

implementation
uses
  SoundManager, TextureManager, Sheduler, TheGame, Player;

{ TMissionBrief }

constructor TMissionBrief.Create;
begin
  Font := TSystemFont.Create('Fixedsys', 24);
  Music1 := SoundMngr.LoadSound('data\music\mission_start.ogg', False);
  Music2 := SoundMngr.LoadSound('data\music\mission_loop.ogg', True);
  Map := TextureMngr.LoadTexture('data\textures\gui\map.jpg', True);

  Rockets[0] := TT11.Create;
  Rockets[0].Pos := CreateVector(0, 1, 0);

  Rockets[1] := TT22.Create;
  Rockets[1].Pos := CreateVector(0, 0.7, 0);

  Rockets[2] := TT33.Create;
  Rockets[2].Pos := CreateVector(0, 0.4, 0);

  Cam := TGLCamera.Create(20);
  Cam.Position := CreateVector(-0.2, 1, 5);
  Cam.Target := CreateVector(-0.2, 1, 0.0);
  SelMiss := 0;

  SoundMngr.Play(Music1);
  Shedulr.SetVarAfter(@Mode, 1, stDword, 6350);

  glClearColor(0, 0, 0, 1);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity;

  gluOrtho2D(0, Screen_Width, Screen_Height, 0);

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity;
end;

function TMissionBrief.GetText: String;
begin
  //
end;

procedure TMissionBrief.KeyDown(Key: Byte);
begin
  if Mode = 2 then
  begin
    if Key = VK_RETURN then
    begin
      SoundMngr.StopAll;
      Game := TGame.Create(TMissileType(SelMiss));
      GameMode := modeGame;
      glEnable(GL_DEPTH_TEST);
    end else
  if Key = vk_down then
  begin
    inc(SelMiss);
    if SelMiss = 3 then SelMiss := 0;
  end else
  if Key = vk_up then
  begin
    Dec(SelMiss);
    if SelMiss = -1 then SelMiss := 2;
  end;
  end;
end;

procedure TMissionBrief.Move;
begin
  Rockets[SelMiss].Bank := Rockets[SelMiss].Bank  + 5; 
end;

procedure TMissionBrief.Render;
var
  i : Integer;
begin
  if Mode = 1 then
  begin
    SoundMngr.Stop(Music1);
    SoundMngr.Play(Music2);
    Mode := 2;
  end;

  OpenGL_DisableTextures;

  if Mode = 0 then
  begin

    glPushMatrix;
    glColor3f(1, 1, 1);
    OpenGL_EnableTextures;
    TextureMngr.Bind(Map);

    glTranslatef(200, Screen_Height - 500, 0);
    glScalef(150, 150, 150);
    glBegin(GL_QUADS);
      glTexCoord2f(0, 1); glVertex2i(0, 0);
      glTexCoord2f(1, 1); glVertex2i(4, 0);
      glTexCoord2f(1, 0); glVertex2i(4, 3);
      glTexCoord2f(0, 0); glVertex2i(0, 3);
    glEnd;

    glPopMatrix;

    OpenGL_DisableTextures;

    OpenGL_EnableBlendining;

    glPushMatrix;

    glTranslatef(350, Screen_Height-250, 0);
    glColor4f(1, 0, 0, sin(elp));

    glPointSize(10);
    glBegin(GL_POINTS);
      glVertex3f(0, 0, 0);
    glEnd;
    glPointSize(1);

    glPopMatrix;

    OpenGL_DisableBlend;
  end else
  if Mode = 2 then
  begin
    OpenGL_DisableBlend;
    {glMatrixMode(GL_PROJECTION);
    glLoadIdentity;
    gluPerspective(90, Screen_Width/Screen_Height, 0.001, 1000);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity;}

    //Cam.DebugMove(Keys, Mouse_Dx, Mouse_Dy);
    Cam.Look;
    glColor3f(1, 1, 1);
    glEnable(GL_DEPTH_TEST);
    //glEnable(GL_LIGHTING);
   // glEnable(GL_LIGHT0);
    for i := 0 to 2 do
    begin
      Rockets[i].Render;
    end;
    //glDisable(GL_LIGHTING);

    //OpenGL_OxysDebug;
    //OpenGL_DepthGridDebug(30);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity;

    gluOrtho2D(0, Screen_Width, Screen_Height, 0);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity;
    Font.Print(200, Screen_Height-500, 'Select missile type:', 0, 255, 0);
    Font.Print(750, 387, 'T-11 x 40');
    Font.Print(750, 520, 'T-22 x 20');
    Font.Print(750, 653, 'T-33 x 10');
  end;

  for i := 0 to High(Strings) do
  begin
      Font.Print(100, 100+i*35, Strings[i], 0, 255, 0);
  end;

  elp := elp + 0.15;

end;

procedure TMissionBrief.SetText(s: String);
begin
  SetLength(Strings, 0);

  while Pos(#13#10, s) <> 0 do
  begin
    SetLength(Strings, Length(Strings)+1);
    Strings[High(Strings)] := Copy(s, 1, Pos(#13#10, s)-1);
    Delete(s, 1, Pos(#13#10, s)+1);
  end;
  SetLength(Strings, Length(Strings)+1);
  Strings[High(Strings)] := S;
end;

end.
