unit Player;

interface
uses
  OpenGL, u_winapi, u_opengl, GLTypes, GameTypes, Projectiles;

type
  TMissileType = (t11, t22, t33);
  TPlayer = class (TGameObject)
  private
    Texture, Bt : GLuint;
    MachineTime : Integer;
//    sndShot : Integer;
    function GetDirection : TGLVector;
  public
    Speed, Alt, Fuel : Single;
    KeyControll,
    EnableBank,
    SpeedUpTilt,
    BackTilt : Boolean;
    MissileType : TMissileType;
    AimTo : TGameObject;
    NormTilt, TiltLimit : Single;
    RocketCnt, RocketTTL : Integer;
    property Direction : TGLVector read GetDirection;
    procedure FireMachine;
    procedure FireRocket;
    constructor Create(RocketType : TMissileType);
    procedure Move; override;
    procedure Render; override;
  private
  end;

implementation
uses
  TextureManager, TheGame, SoundManager;

{ TPlayer }

constructor TPlayer.Create(RocketType : TMissileType);
begin
  Pos.X := 0;
  Pos.Z := 0;
  Pos.Y := 0;
  TypeName := 'Player';
  LoadModel('data\meshes\f14.scwm');
  Texture := TextureMngr.LoadTexture('data\textures\military.jpg', True);
  Bt := TextureMngr.LoadTexture('data\textures\sprite\bullet.tga', True);
  NormTilt := 0;
  SpeedUpTilt := True;
  BackTilt := True;
  TiltLimit := 70;
  //sndShot := SoundMngr.LoadSound('data\sfx\shot.ogg', False);
  Fuel := 100;
  MissileType := RocketType;

  case RocketType of
    t11 : RocketCnt := 40;
    t22 : RocketCnt := 20;
    t33 : RocketCnt := 10;
  end;

  RocketTTL := RocketCnt;
end;

procedure TPlayer.FireMachine;
{var
  bullet : TBullet;}
begin
  {if MachineTime = 0 then
  begin
    MachineTime := 10;
    //SoundMngr.Play(sndShot);
    bullet := TBullet.Create(Bt);
    bullet.Pos := Pos;
    bullet.Pos.Y := bullet.Pos.Y;

    bullet.Dir := NormalizeVector( SubVectors(Pos, Game.HUD.Target_p ) );
    bullet.FiredBy := Self;
    
    bullet.Speed := Speed;
    bullet.Size := 0.03;
    Game.AddObject(bullet);
  end;    }
end;

procedure TPlayer.FireRocket;
var
  Rock : TProjectile;
begin
  if RocketCnt = 0 then Exit;
  
  if AimTo <> nil then
  begin

    case MissileType of
      t11 : Rock := TT11.Create;
      t22 : Rock := TT22.Create;
      t33 : Rock := TT33.Create;
    else
      Rock := TT11.Create;
    end;

    Rock.Pos := Pos;
    Rock.Bank := Bank;
    Rock.HAng := HAngle;
    Rock.VAng := Tilt;
    Rock.AimTo := AimTo;
    Rock.Speed := Speed + 0.5;
    Rock.FiredBy := Self;

    Game.AddObject(Rock);
    Dec(RocketCnt);
  end;
end;

function TPlayer.GetDirection: TGLVector;
begin
  Result := Trigon3DVector(HAngle+90, -Tilt);
end;

procedure TPlayer.Move;
var
  TiltFactor : Single;
begin

  if MachineTime > 0 then Dec(MachineTime);

  if KeyControll then
  begin
    if Keys[ord('W')] then
    begin
      Tilt := Tilt + 0.5;
    end else
    if Keys[ord('S')] then
    begin
      Tilt := Tilt - 0.5;
    end else
    begin
      if BackTilt then
      begin
        if Tilt > NormTilt then Tilt := Tilt - 0.1;
        if Tilt < NormTilt then Tilt := Tilt + 0.1;
      end;
    end;

    //if Keys[ord('Q')] then FireMachine;

    if EnableBank then
    begin
      if Keys[ord('A')] then Bank := Bank - 0.5 else
      if Keys[ord('D')] then Bank := Bank + 0.5 else
      begin
        if Bank > 0 then Bank := Bank - 0.5;
        if Bank < 0 then Bank := Bank + 0.5;
      end;

      if Bank > 35 then Bank := 35;
      if Bank < - 35 then Bank := -35;
    end else
    begin
      if Keys[ord('A')] then
      begin
        Pos := SubVectors(Pos, MultyplyVectorScalar(Trigon3DVector(HAngle+180, Tilt), 0.5));
      end else
      if Keys[ord('D')] then
      begin
        Pos := AddVectors(Pos, MultyplyVectorScalar(Trigon3DVector(HAngle-180, Tilt), 0.5));
      end;
    end;

    if Tilt > TiltLimit then Tilt := TiltLimit;
    if Tilt < -TiltLimit then Tilt := -TiltLimit;

    if SpeedUpTilt then
    begin
      TiltFactor := Sin(Tilt / (180 / pi));

      Alt := Alt - TiltFactor * 0.03;

      Speed := Speed + TiltFactor * 0.0005;
    end;

    if Speed < 0.01 then
      Speed := 0.01;

    if Speed > 1.8 then
      Speed := 1.8;

    HAngle := HAngle + Sin(Bank / (180/pi)) * 0.5;

    Fuel := Fuel - 0.01;
    if Fuel < 0 then Fuel := 0;
  end;

  Pos := AddVectors(Pos, MultyplyVectorScalar(Direction, Speed));
  Pos.Y := Alt;
end;

procedure TPlayer.Render;
begin
  glPushMatrix;

  OpenGL_Translate(Pos);

  glColor3f(0, 1, 0);
  glScalef(0.2, 0.2, 0.2);

  glRotatef(180-HAngle, 0.0, 1.0, 0.0);
  glRotatef(-Tilt, 1.0, 0.0, 0.0);
  glRotatef(-Bank, 0.0, 0.0, 1.0);

  OpenGL_EnableTextures;
  TextureMngr.Bind(Texture);

  glBegin(GL_TRIANGLES);
  inherited;
  glEnd;

  OpenGL_DisableTextures;

  glPopMatrix;
end;

end.
