unit Projectiles;

interface
uses
  OpenGL, Windows, u_winapi, u_opengl, GameTypes, GLTypes, Billboards;

type

  TProjectile = class (TGameObject)
  private
    Texture : Integer;
    SmokeT : Integer;
    
  public
    Dir : TGLVector;
    Bank, HAng, VAng, Speed : Single;
    AimTo : TGameObject;
    FiredBy : TGameObject;
    TTL : Integer;
    procedure Render; override;
    procedure Move; Override;
    constructor Create; virtual;
  end;

  TT11 = class (TProjectile)
  public
    constructor Create; override;
    procedure Move; override;
  end;

  TT22 = class (TProjectile)
  public
    constructor Create; override;
    procedure Move; override;
  end;

  TT33 = class (TProjectile)
  public
    constructor Create; override;
    procedure Move; override;
  end;

  TBullet = class (TBillboard)
    Dir : TGLVector;
    FiredBy : TGameObject;
    TTL : Integer;
    Speed : Single;
    procedure Render; override;
    constructor Create(tId : Integer);
    procedure Move; override;
  end;



implementation
uses
  TextureManager, TheGame;

{ TT11 }

constructor TT11.Create;
begin
  LoadModel('data\meshes\t11.scwm');
  inherited;
end;

procedure TT11.Move;
begin
  inherited;

end;

{ TT22 }

constructor TT22.Create;
begin
  LoadModel('data\meshes\t22.scwm');
  inherited;
end;

procedure TT22.Move;
begin
  inherited;

end;

{ TT33 }

constructor TT33.Create;
begin
  LoadModel('data\meshes\t33.scwm');
    inherited;
end;

procedure TT33.Move;
begin
  inherited;

end;

{ TProjectile }

constructor TProjectile.Create;
begin
  Texture := TextureMngr.LoadTexture('data\textures\rocket.jpg', True);
  SmokeT := TextureMngr.LoadTexture('data\textures\sprite\smoke.tga', True);
  Collision := True;
  Radius := 1;
  TypeName := 'Rocket';
end;

procedure TProjectile.Move;
var
  Smoke : TSmokeBBD;
begin
  if AimTo <> nil then
  begin
    Dir := CreateVector(AimTo.Pos.X - Pos.X, AimTo.Pos.Y - Pos.Y, AimTo.Pos.Z - Pos.Z);
    Dir := NormalizeVector(Dir);

    HAng := -AngleTo(Pos.X, Pos.Z, AimTo.Pos.X, AimTo.Pos.Z);
  end;

  Pos := AddVectors(Pos, MultyplyVectorScalar( Dir, Speed) );

  Smoke := TSmokeBBD.Create(SmokeT, 0.1);
  Smoke.Pos := Pos;
  if AimTo <> nil then
    Game.AddObject(Smoke, True) else
      Game.AddObject(Smoke);

  inc(TTL);

  if TTL = 2000 then MustDie := True;
end;

procedure TProjectile.Render;
begin
  glPushMatrix;
  OpenGL_Translate(Pos);


  glRotatef(HAng, 0.0, 1.0, 0.0);
  glRotatef(VAng, 0.0, 0.0, 1.0);
  glRotatef(Bank, 1.0, 0.0, 0.0);

  glScalef(0.1, 0.1, 0.1);
  OpenGL_EnableTextures;
  TextureMngr.Bind(Texture);
  glColor3f(1, 1, 1);
  glBegin(GL_TRIANGLES);
  inherited;
  glEnd;
  OpenGL_DisableTextures;
  glPopMatrix;
end;

{ TBullet }

constructor TBullet.Create(tId: Integer);
begin
  inherited;
  Self.tId := tId;
  Collision := True;
  Radius := 0.1;
end;

procedure TBullet.Move;
begin
  inherited;
  Pos := AddVectors(Pos, MultyplyVectorScalar(Dir, Speed+0.5));
  inc(TTL);

  if TTL >= 500 then MustDie := True;
end;

procedure TBullet.Render;
begin
  inherited;
end;

end.
