unit SoundManager;

interface
uses
  Windows, bass;

type
  TSoundMessageType = (mError, mInfo);
  TSoundManager = class
  private
    Sounds : array of record
      Name : String;
      Handle : HSTREAM;
    end;
    WinHand : HWND;
    procedure Log(Mess : String; mType : TSoundMessageType);
  public
    Enabled : Boolean;
    LogHandler : procedure(s : String);
    constructor Create(WinID : HWND);
    function LoadSound(const FileName : String; Loop : boolean = false):Integer;
    procedure Play(ID : Integer);
    procedure StopAll;
    procedure Stop(ID : Integer);
    procedure Pause(ID : Integer);
    procedure PauseAll;
  end;

Var
  SoundMngr : TSoundManager;

implementation

{ TSoundManager }

constructor TSoundManager.Create(WinID : HWND);
begin
  Enabled := True;
  WinHand := WinID;
  if (HIWORD(BASS_GetVersion) <> BASSVERSION) then
  begin
    Log('Invalid BASS API Version!', mError);
    Exit;
  end;

  if not BASS_Init(-1, 44100, 0, WinID, nil) then
  begin
    Log('Can not init. bass.dll!', mError);
	  Exit;
  end;
end;

function TSoundManager.LoadSound(const FileName: String;
  Loop: boolean): Integer;
var
  I:Integer;
begin
  Result := 0;
  if not Enabled then Exit;

  For I := 0 to High(Sounds) do
  begin
    if Sounds[I].Name = FileName  then
    begin
      Result := I;
      Exit;
    end;
  end;

  SetLength(Sounds, Length(Sounds)+1);
  Result := High(Sounds);
  Sounds[Result].Name := FileName;
  Sounds[Result].Handle := BASS_StreamCreateFile(False, PChar(FileName), 0, 0, 0);

  if Loop then
    BASS_ChannelFlags(Sounds[Result].Handle, BASS_SAMPLE_LOOP, BASS_SAMPLE_LOOP);

  Log(FileName, mInfo);
end;

procedure TSoundManager.Log(Mess: String; mType: TSoundMessageType);
var
  pFix : String;
begin
  case mType of
    mError : pFix := 'SoundManager error: ';
    mInfo  : pFix := 'SoundManager: ';
  end;

  if @LogHandler <> nil then
    LogHandler(pFix + Mess);

  if mType = mError then
    MessageBox(WinHand, PChar(Mess), PChar(pFix), MB_ICONERROR);
end;

procedure TSoundManager.Pause(ID: Integer);
begin
  if not Enabled then Exit;
  BASS_ChannelPause(Sounds[ID].Handle);
end;

procedure TSoundManager.PauseAll;
var
  I:Integer;
begin
  if not Enabled then Exit;
  for I := 0 to High(Sounds) do
    Pause(I);
end;

procedure TSoundManager.Play(ID: Integer);
begin
  if not Enabled then Exit;
  BASS_ChannelPlay(Sounds[ID].Handle, True);
end;

procedure TSoundManager.Stop(ID: Integer);
begin
  if not Enabled then Exit;
  BASS_ChannelStop(Sounds[ID].Handle);
end;

procedure TSoundManager.StopAll;
var
  I:Integer;
begin
  if not Enabled then Exit;
  for I := 0 to High(Sounds) do
    Stop(I);
end;

end.
