unit glCamera;

interface
uses
  OpenGL, GLTypes;

type
  TGLCamera = class
    Position : TGLVector;
    Target   : TGLVector;
    cfSpeed    : GLfloat;
    Bank     : GLfloat;
    AHor, AVert : Single;
  private
    aFov, nFov, Aspect : GLfloat;
    function GetFov:GLfloat;
    procedure SetFov(val : GLfloat);

    function GetAspectRatio:GLfloat;
    procedure SetAspectRation(val : GLfloat);

    function GetActualFov : GLfloat;
    procedure SetActualFov (af : GLfloat);
  public
    property Fov:GLfloat read GetFov write SetFov;
    property AspectRatio:GLfloat read GetAspectRatio write SetAspectRation;
    property ActualFov: GLfloat read GetActualFov write SetActualFov;

    procedure SetPosition(x,y,z:GLfloat);
    procedure SetTarget(x,y,z:GLfloat);
    procedure Look;
    procedure Move;
    procedure DebugMove(Keys : array of Boolean; Dx, Dy : Integer);
    constructor Create(cFov : Single = 60; nAspect:GLfloat = 4/3);
  end;

implementation

uses Math;

{ TGLCamera }

constructor TGLCamera.Create(cFov : Single = 60; nAspect:GLfloat = 4/3);
begin
  aFov := cFov;
  nFov := cFov;
  cfSpeed := 0.5;
  Aspect := nAspect;
  AHor := 0;
  AVert := 0;
end;

procedure TGLCamera.DebugMove(Keys: array of Boolean; Dx, Dy : Integer);
var
  DirV : TGLVector;
begin
  AHor := AHor + Dx * 0.3;
  AVert := AVert - Dy * 0.3;

  DirV := Trigon3DVector(AHor, AVert);

  if Keys[ord('W')] then
    Position := AddVectors(Position, MultyplyVectorScalar(DirV, 0.1));
  if Keys[ord('S')] then
    Position := SubVectors(Position, MultyplyVectorScalar(DirV, 0.1));
  if Keys[ord('A')] then
    Position := AddVectors(Position, MultyplyVectorScalar(Trigon3DVector(AHor-90, AVert), 0.1));
  if Keys[ord('D')] then
    Position := AddVectors(Position, MultyplyVectorScalar(Trigon3DVector(AHor+90, AVert), 0.1));

  Target :=  AddVectors(Position, DirV);

end;

function TGLCamera.GetActualFov: GLfloat;
begin
  Result := aFov;
end;

function TGLCamera.GetAspectRatio: GLfloat;
begin
  Result := Aspect;
end;

function TGLCamera.GetFov: GLfloat;
begin
  Result := aFov;
end;

procedure TGLCamera.Look;
begin
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity;
  gluPerspective(aFov, Aspect, 0.01, 10000.0);

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity;

  glRotatef(Bank, 0.0, 0.0, 1.0);
  gluLookAt(Position.X, Position.Y, Position.Z,
            Target.X,   Target.Y,   Target.Z,
            0,          1,          0);

end;

procedure TGLCamera.Move;
begin
  if aFov < nFov then
    aFov := aFov + cfSpeed else
    if aFov > nFov then
    aFov := aFov - cfSpeed;
end;

procedure TGLCamera.SetActualFov(af: GLfloat);
begin
  aFov := af;
  nFov := af;
end;

procedure TGLCamera.SetAspectRation(val: GLfloat);
begin
  Aspect := val;
end;

procedure TGLCamera.SetFov(val: GLfloat);
begin
  nFov := val;
end;

procedure TGLCamera.SetPosition(x, y, z: GLfloat);
begin
  Position.X := x;
  Position.Y := y;
  Position.Z := z;
end;

procedure TGLCamera.SetTarget(x, y, z: GLfloat);
begin

end;

end.
