unit uAmbience;

interface
uses
  OpenGL, GameTypes, TextureManager, u_opengl, glCamera, GLTypes;
type
  TAmbienceType = (abDay);
  TWater = class (TGameObject)
  private
    Texture : Integer;
  public
    procedure Render; override;
    procedure Move; override;
    constructor Create;
  end;

  TAmbience = class (TGameObject)
  private
    SkyTexture : GLuint;
    Ground : TGameObject;
    Camera : TGLCamera;
  public
    procedure Move; override;
    procedure Render; override;
    constructor Create(AType : TAmbienceType; Camer : TGLCamera);
  end;

const
  FLOOR_SIZE = 2000;
  WATER_TILE = 50;

implementation

{ TAmbience }

constructor TAmbience.Create(AType: TAmbienceType; Camer : TGLCamera);
var
  fog_color : array[0..3] of GLfloat;
begin
  Camera := Camer;
  LoadModel('data\meshes\dome.scwm');
  case AType of
    abDay : begin
              fog_color[0] := 1;
              fog_color[1] := 1;
              fog_color[2] := 1;
              fog_color[3] := 1;

              glEnable(GL_FOG);
              glFogi(GL_FOG_MODE, GL_EXP2);
              glFogfv(GL_FOG_COLOR, @fog_color);
              glFogf(GL_FOG_DENSITY, 0.05);

              SkyTexture := TextureMngr.LoadTexture('data\textures\sky_day.jpg');
            end;
  end;
  Ground := TWater.Create;
end;

procedure TAmbience.Move;
begin
  Ground.Move;
end;

procedure TAmbience.Render;
var
  i,j:Shortint;
begin
  for i := -1 to 1 do for j := -1 to 1 do
  begin
    Ground.Pos.X := ((Trunc(Camera.Position.X) div FLOOR_SIZE) + i) * FLOOR_SIZE;
    Ground.Pos.Z := ((Trunc(Camera.Position.Z) div FLOOR_SIZE) + j) * FLOOR_SIZE;
    Ground.Render;
  end;

  glDisable(GL_FOG);
  OpenGL_EnableTextures;
  TextureMngr.Bind(SkyTexture);
  glPushMatrix;
  glTranslatef(Camera.Position.X, 0, Camera.Position.Z);
  glScalef(1000, 1000, 1000);
  glColor3f(1, 1, 1);
  glBegin(GL_TRIANGLES);
  inherited;
  glEnd;
  OpenGL_DisableTextures;
  //glEnable(GL_FOG);
  glPopMatrix;
end;

{ TWater }

constructor TWater.Create;
begin
  Texture := TextureMngr.LoadTexture('data\textures\water.jpg', True);
end;

procedure TWater.Move;
begin

  {TexMove := TexMove + 0.01;
  if TexMove > 1 then TexMove := 0;}

end;

procedure TWater.Render;
begin
  glPushMatrix;

  glTranslatef(Pos.X, Pos.Y, Pos.Z);

  OpenGL_EnableTextures;
  TextureMngr.Bind(Texture);

  glColor3f(1, 1, 1);
    glBegin(GL_QUADS);
      glTexCoord2f(WATER_TILE,  0);             glVertex3f(-FLOOR_SIZE div 2,   0,  -FLOOR_SIZE div 2);
      glTexCoord2f(0,           0);             glVertex3f( FLOOR_SIZE div 2,   0,  -FLOOR_SIZE div 2);
      glTexCoord2f(0,           WATER_TILE);    glVertex3f( FLOOR_SIZE div 2,   0,   FLOOR_SIZE div 2);
      glTexCoord2f(WATER_TILE,  WATER_TILE);    glVertex3f(-FLOOR_SIZE div 2,   0,   FLOOR_SIZE div 2);
    glEnd;
  glEnd;
  OpenGL_DisableTextures;
  glPopMatrix;
end;

end.
