unit uGameObjects;

interface
uses
  OpenGL, u_opengl, GameTypes, GLTypes, Player;

type
  TCarrier = class (TGameObject)
  private
    Texture : Integer;
  public
    constructor Create;
    procedure Move; override;
    procedure Render; override;
  end;

  TC135 = class (TGameObject)
  private
    Texture,
    Shlang_t,
    metall : Integer;
    Shaking : Single;
    mov     : Single;
    
    Connector : GLuint;
  public
    Player : TPlayer;
    Shlang  : Single;
    Away : Boolean;
    constructor Create(Play : TPlayer);
    procedure Move; override;
    procedure Render; override;
  end;

  TMigAttackType = (atFlyBy, atFlyTo, atFlyToMissile);

  TMig28 = class (TGameObject)
  private
    AttackType : TMigAttackType;
    Player : TPlayer;
    Dir : TGLVector;
    Texture : GLuint;
    Flyed, Fired : Boolean;
    sndImpact, TTL : Integer;
  public
    Speed : Single;
    constructor Create(Player : TPlayer; AType : TMigAttackType; cnt : Integer = 0);
    procedure Render; override;
    procedure Move; override;
    procedure DoCollision(CollisionedWith : TObject); override;
  end;

implementation
uses
  TextureManager, uModelManager, SoundManager, TheGame, Projectiles;

{ TCarrier }

constructor TCarrier.Create;
begin
  TypeName := 'Carrier';
  LoadModel('data\meshes\carrier.scwm');
  Texture := TextureMngr.LoadTexture('data\textures\carrier.jpg', True);
end;

procedure TCarrier.Move;
begin
  inherited;
end;

procedure TCarrier.Render;
begin
  glPushMatrix;
  OpenGL_Translate(Pos);
  OpenGL_EnableTextures;
  TextureMngr.Bind(Texture);

  glRotatef(Bank,   0.0, 0.0, 1.0);
  glRotatef(HAngle, 0.0, 1.0, 0.0);
  glRotatef(Tilt,   1.0, 0.0, 0.0);

  glColor3f(1.0, 1.0, 1.0);
  glBegin(GL_TRIANGLES);
  inherited;
  glEnd;
  OpenGL_DisableTextures;
  glPopMatrix;
end;

{ TC135 }

constructor TC135.Create(Play : TPlayer);
begin
  TypeName := 'C135';
  LoadModel('data\meshes\c135.scwm');

  Connector := ModelManager.LoadModelToGLList('data\meshes\connector.scwm');

  Texture := TextureMngr.LoadTexture('data\textures\military.jpg', True);
  Shlang_t := TextureMngr.LoadTexture('data\textures\shlang.tga', True);
  metall := TextureMngr.LoadTexture('data\textures\metall.jpg', True);

  Player := Play;
  HAngle := Player.HAngle;
  mov := -10;
  Shlang := 0;
end;

procedure TC135.Move;
begin

  if Not Away then
  begin

    HAngle := Player.HAngle;
    Pos := AddVectors(Player.Pos, MultyplyVectorScalar(Player.Direction, mov));
    Pos.Y := Pos.Y + 4 + sin(Shaking)*0.1;
    Pos.X := Pos.X + cos(Shaking)*0.1;
    Pos.Z := Pos.Z - cos(Shaking)*0.1;

    if mov < 25 then
      mov := mov + 0.03 else
        if Shlang < 15 then
          Shlang := Shlang + 0.02;
  end else
  begin

    Pos := AddVectors(Player.Pos, MultyplyVectorScalar(Player.Direction, mov));
    Pos.Y := Pos.Y + 4 + sin(Shaking)*0.1;
    Pos.X := Pos.X + cos(Shaking)*0.1;
    Pos.Z := Pos.Z - cos(Shaking)*0.1;

    if Shlang > 0 then
      Shlang := Shlang - 0.02 else
        mov := mov + 0.05;
  end;
  Shaking := Shaking + 0.01;
end;

procedure TC135.Render;
begin
  glPushMatrix;

  OpenGL_Translate(Pos);
  glRotatef(-HAngle, 0.0, 1.0, 0.0);
  OpenGL_OxysDebug;

  glPushMatrix;
  glColor4f(1, 1, 1, 1);
  glTranslatef(0.0, 1.0, -7);
  glRotatef(-17, 1.0, 0.0, 0.0);
  glRotatef(45, 0.0, 0.0, 1.0);

  OpenGL_EnableTextures;
  TextureMngr.Bind(Shlang_t);
  glBegin(GL_QUADS);
    glTexCoord2f(1, 0); glVertex3f(-0.1, 0,  0);
    glTexCoord2f(0, 0); glVertex3f( 0.1, 0,  0);
    glTexCoord2f(0, 1); glVertex3f( 0.1, 0, -Shlang);
    glTexCoord2f(1, 1); glVertex3f(-0.1, 0, -Shlang);

    glTexCoord2f(1, 0); glVertex3f( 0, -0.1,  0);
    glTexCoord2f(0, 0); glVertex3f( 0,  0.1,  0);
    glTexCoord2f(0, 1); glVertex3f( 0,  0.1, -Shlang);
    glTexCoord2f(1, 1); glVertex3f( 0, -0.1, -Shlang);
  glEnd;


  TextureMngr.Bind(metall);
  glTranslatef(0.0, 0.0, -Shlang);
  glRotatef(90, 1.0, 0.0, 0.0);
  glRotatef(45, 0.0, 1.0, 0.0);
  glScalef(0.1, 0.1, 0.1);
  glBegin(GL_TRIANGLES);
    glCallList(Connector);
  glEnd;

  glPopMatrix;

  OpenGL_EnableTextures;
  TextureMngr.Bind(Texture);
  glScalef(0.5, 0.5, 0.5);
  glTranslatef(-16.5, 0.0, 0.0);
  glBegin(GL_TRIANGLES);
  inherited;
  glEnd;


  OpenGL_DisableTextures;
  glPopMatrix;
end;

{ TMig28 }

constructor TMig28.Create(Player: TPlayer; AType: TMigAttackType; cnt : Integer = 0);
begin

  TypeName := 'Mig 28';

  Collision := True;
  Radius := 2;
  Health := 100;
  
  LoadModel('data\meshes\mig28.scwm');
  sndImpact := SoundMngr.LoadSound('data\sfx\impact.ogg', False);
  Texture := TextureMngr.LoadTexture('data\textures\mig28.jpg', True);

  AttackType := AType;
  Self.Player := Player;

  Dir := Player.Direction;
  Dir.Y := 0;
  Dir := NormalizeVector(Dir);

  case AType of
    atFlyBy :        begin
                      Pos := AddVectors(Player.Pos, MultyplyVectorScalar(Dir, -50 - cnt * 30));
                      //Dir := MultyplyVectorScalar(Dir, -1);
                     end;
    atFlyTo :        begin
                      Pos := AddVectors(Player.Pos, MultyplyVectorScalar(Dir, 100 + cnt * 30));
                     end;
    atFlyToMissile : begin
                       Pos := AddVectors(Player.Pos, MultyplyVectorScalar(Dir, 100 + cnt * 30));
                     end;
  end;

  if AType <> atFlyBy then
  begin
    Pos.Y := Player.Alt + 1 + cnt * 1.5;
  end else
    Pos.Y := Player.Alt - 1 - cnt * 1.5;  

  HAngle := AngleTo(Pos.X, Pos.Z, Player.Pos.X, Player.Pos.Z) - 90;
end;

procedure TMig28.DoCollision(CollisionedWith: TObject);
begin
  inherited;
  if (CollisionedWith is TProjectile) and ((CollisionedWith as TProjectile).FiredBy <> Self) then
  begin
    MustDie := True;
    (CollisionedWith as TProjectile).MustDie := True;
    SoundMngr.Play(sndImpact);
  end else
  if (CollisionedWith is TBullet) and ((CollisionedWith as TBullet).FiredBy <> Self) then
  begin

  end;
end;

procedure TMig28.Move;
var
  R : TT11;
begin
  if not Flyed and (Distance(Pos, Player.Pos) < 20) then
  begin
    Flyed := True;
    SoundMngr.Play(Game.sndFlyBy);
  end;

  case AttackType of
    atFlyBy :        begin
                      Pos := AddVectors(Pos, MultyplyVectorScalar(Dir, Player.Speed + 0.3));

                      if Flyed then Pos.Y := Pos.Y + 0.05;
                      TTL := TTL + 1;
                      if TTL >= 800 then MustDie := True;
                     end;
    atFlyTo :        begin
                      if Speed > -1 then
                        Speed := Speed - 0.001 else MustDie := True;
                        
                      Pos := AddVectors(Pos, MultyplyVectorScalar(Dir, Player.Speed + Self.Speed));
                     end;
    atFlyToMissile : begin
                      if Speed > -0.8 then
                        Speed := Speed - 0.001 else MustDie := True;

                      if not Fired and (Distance(Pos, Player.Pos) < 60) then
                      begin
                        Fired := True;
                        R := TT11.Create;
                        R.HAng := -AngleTo(Pos.X, Pos.Z, Player.Pos.X, Player.Pos.Z);
                        R.Dir := NormalizeVector(SubVectors(Player.Pos, Pos));
                        //Trigon3DVector(R.HAng, AngleTo(Pos.X, Pos.Y, Player.Pos.X, Player.Pos.Y));
                        
                        R.Speed := Player.Speed + Self.Speed + 0.3;
                        R.AimTo := nil;
                        R.Pos := AddVectors(Pos, MultyplyVectorScalar(Dir, 1.2));
                        R.FiredBy := Self;
                        Game.AddObject(R);
                      end;
                        
                      Pos := AddVectors(Pos, MultyplyVectorScalar(Dir, Player.Speed + Self.Speed));
                     end;
  end;
end;

procedure TMig28.Render;
begin
  glPushMatrix;
  OpenGL_Translate(Pos);
  glScalef(0.1, 0.1, 0.1);
  
  glRotatef(180-HAngle, 0.0, 1.0, 0.0);
  glRotatef(-Tilt, 1.0, 0.0, 0.0);
  glRotatef(-Bank, 0.0, 0.0, 1.0);

  glColor3f(1, 1, 1);

  OpenGL_EnableTextures;
  TextureMngr.Bind(Texture);
  glBegin(GL_TRIANGLES);
  inherited;
  glEnd;

  OpenGL_DisableTextures;
  glPopMatrix;
end;

end.
