unit uModelManager;

interface
uses
  OpenGL, GLTypes;

type
  TModelManager = class
  private
    Models : array of record
      FileName : String;
      List : GLuint;
    end;
  public
    Log : procedure (s : String);
    function LoadModelToGLList(const FileName : String):GLuint;
  end;

var
  ModelManager : TModelManager;

implementation
type
  {Header for *.SCWM (Simple ClockWork Model}
  TFileHeader = record
    magic  : Cardinal; //4D574353h
    nverts : Integer;
    nface  : Integer;
  end;

{ TModelManager }

function TModelManager.LoadModelToGLList(const FileName: String): GLuint;
type
  TVertexes = array[0..0] of TGLVector;
  TFaces = array[0..0] of TGLFace;
  TTexCoords = array[0..0] of TGLTexCoord;
var
  FileHeadr : TFileHeader;
  f : File of Byte;
  VertexData, FacesData, TCoordsData : Pointer;
  I : Integer;
begin
  for i := 0 to high(Models) do
  begin
    if Models[i].FileName = FileName then
    begin
      Result := Models[i].List;
      Exit;
    end;
  end;

  AssignFile(f, FileName);
  Reset(f);

  BlockRead(f, FileHeadr, SizeOf(TFileHeader));

  GetMem(VertexData, FileHeadr.nverts * SizeOf(TGLVector));
  BlockRead(f, VertexData^, FileHeadr.nverts * SizeOf(TGLVector));

  GetMem(FacesData, FileHeadr.nface * SizeOf(TGLFace));
  BlockRead(f, FacesData^, FileHeadr.nface * SizeOf(TGLFace));

  GetMem(TCoordsData, FileHeadr.nverts * SizeOf(TGLTexCoord));
  BlockRead(f, TCoordsData^, FileHeadr.nverts * SizeOf(TGLTexCoord));

  CloseFile(f);

  Result := glGenLists(1);
  glNewList(Result, GL_COMPILE);

  for I := 0 to FileHeadr.nface-1 do
  begin
    glTexCoord2fv(@TTexCoords(TCoordsData^)[TFaces(FacesData^)[i].v1]); glVertex3fv(@TVertexes(VertexData^)[TFaces(FacesData^)[i].v1]);
    glTexCoord2fv(@TTexCoords(TCoordsData^)[TFaces(FacesData^)[i].v2]); glVertex3fv(@TVertexes(VertexData^)[TFaces(FacesData^)[i].v2]);
    glTexCoord2fv(@TTexCoords(TCoordsData^)[TFaces(FacesData^)[i].v3]); glVertex3fv(@TVertexes(VertexData^)[TFaces(FacesData^)[i].v3]);
  end;

  glEndList;

  SetLength(Models, Length(Models)+1);
  Models[High(Models)].FileName := FileName;
  Models[High(Models)].List := Result;

  if @Log <> nil then
    Log('Model Manager: ' + FileName);
end;

end.
