unit u_opengl;

interface
uses
  Windows,
  OpenGL, GLTYpes;

var
  DC : HDC;
  RC : HGLRC;

function OpenGL_Init(H_Wnd: HWND):Boolean;
procedure OpenGL_OxysDebug;
procedure OpenGL_DepthGridDebug(side:Integer);
procedure OpenGL_BindTexture(ID : GLuint);
procedure OpenGL_EnableTextures;
procedure OpenGL_DisableTextures;
procedure OpenGL_EnableBlendining;
procedure OpenGL_DisableBlend;
procedure OpenGL_Translate(v : TGLVector);

procedure glBindTexture(target: GLenum; texture: GLuint); stdcall; external opengl32;
function gluBuild2DMipmaps(Target: GLenum; Components, Width, Height: GLint; Format, atype: GLenum; Data: Pointer): GLint; stdcall; external glu32;
procedure glGenTextures(n: GLsizei; var textures: GLuint); stdcall; external opengl32;

implementation

procedure OpenGL_DisableBlend;
begin
  glDisable(GL_BLEND);
end;

procedure OpenGL_EnableBlendining;
begin
  glEnable(GL_BLEND);
glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
end;

procedure OpenGL_Translate(v : TGLVector);
begin
  glTranslatef(v.X, v.Y, v.Z);
end;

procedure OpenGL_EnableTextures;
begin
  glEnable(GL_TEXTURE_2D);
end;

procedure OpenGL_DisableTextures;
begin
  glDisable(GL_TEXTURE_2D);
end;

procedure OpenGL_BindTexture(ID : GLuint);
begin
  glEnable(GL_TEXTURE_2D);
  glBindTexture(GL_TEXTURE_2D, ID);
end;

procedure OpenGL_DepthGridDebug;
var
  i,j:Integer;
begin
  glPushMatrix;
  glColor3f(0.0, 0.7, 0.58);

  for i := -(side div 2) to (side div 2) do
  begin
    for j := -(side div 2) to (side div 2) do
    begin
      glBegin(GL_LINE_LOOP);
        glVertex3f(i, 0, j);
        glVertex3f(i+1, 0, j);
        glVertex3f(i+1, 0, j+1);
        glVertex3f(i, 0, j+1);
      glEnd;
    end;
  end;
  glPopMatrix;
end;

procedure OpenGL_OxysDebug;
begin
  glBegin(GL_LINES);

    glColor3f(1.0, 0.0, 0.0);
    glVertex3f(0.0, 0.0, 0.0);
    glVertex3f(1.0, 0.0, 0.0);

    glColor3f(0.0, 1.0, 0.0);
    glVertex3f(0.0, 0.0, 0.0);
    glVertex3f(0.0, 1.0, 0.0);

    glColor3f(0.0, 0.0, 1.0);
    glVertex3f(0.0, 0.0, 0.0);
    glVertex3f(0.0, 0.0, 1.0);
  glEnd;
end;

function OpenGL_Init;
var
  pfd : TPixelFormatDescriptor; //   
  n   : Integer;
begin
  DC := GetDC(H_Wnd);
  with pfd do //  
  begin
    nSize      := SizeOf(pfd);
    nVersion   := 1;
    dwFlags    := PFD_DOUBLEBUFFER or //  
                  PFD_SUPPORT_OPENGL; //  OpenGL
    iPixelType := PFD_TYPE_RGBA;      //    RGBA
    cColorBits := 24;               //  
    cDepthBits := 16;                  //  Depth  (aka Z-Bufer);
  end;

  n := ChoosePixelFormat(DC, @pfd);   //    
  SetPixelFormat(DC, n, @pfd);        //    
  DescribePixelFormat(DC, n, sizeof(pfd), pfd);

  RC := wglCreateContext(DC);         //   
  Result := RC<>0;

  if Result then
    wglMakeCurrent(DC, RC);           //     
  glClearColor(1, 1, 1, 1);
  glDisable(GL_CULL_FACE);
end;

end.
