unit _Objects;

interface

Uses
  Use_Geometry,
  Use_Parts,
  Use_Text,
  OpenGL,
  Global;

Const
  Max_Bullets = 1000;
  Max_Buildings = 10;

type

 TGame_State = (gs_none,
                gs_action,
                gs_menu,
                gs_help,
                gs_intro);

 T_Bullet_Tip = (BT_None,
                 BT_Rocket,
                 BT_Sputnick,
                 BT_Bullet,
                 BT_Anti_Rocket,
                 BT_Bot);

 TCity = record
    Poz:TVect;
    Size:single;
    tip:byte;
    life:byte;{?}
 end;

 TBullet = record {Bullet,Rocket etc..}
    Point:TVect;
    Vect:TVect;
    Cel:TVect;
    Ang:Single;
    Speed:single;
    act:byte;
    Tip:T_Bullet_Tip;
 end;

 TBuild = record {Bullet,Rocket etc..}
    Point:TVect;
    life:byte;
    Tip:T_Bullet_Tip;
 end;

 TWorld = class
  private
   finded:boolean;
   City:Array[0..5]of TCity;
   Out_Fire,out_city:byte;
   {StarFields BackGround pozition(2 fields)}
   SF1,SF2:TVect;
   procedure Play_Sound(Sound:integer);
   procedure Put_Sprite(Point:TVect;Ang,Mash,Size:Single);Overload;
   procedure Put_Sprite(Point:TVect;Mash,Size:Single);Overload;
   procedure Put_Tile(Point:TVect;Mash,Size:Single;a,b:byte);
   procedure Put_Tile_4(Point:TVect;Ang,Mash,Size:Single;a,b:byte);

   procedure Put_Bullet(B:TBullet);
   procedure Draw_StarField;
   Procedure Render_Menu;
   Procedure Render_Scene;
   procedure Find_Coord;    
   procedure Boom(P:TVect);
   procedure poom(P:TVect);
  public
   SF1_Tex,
   SF2_Tex,
   City_Tex,
   Bullets_Tex:Cardinal;
   {game scenario}
   Scene:string;
   tact:integer;
   level:byte;
   {game_vars}
   Game_State:TGame_State;
   //Level_num:byte;
   {screen_props}
   Mash:single;{}
   Stress:byte;
   K:TVect;{   KX,KY}
   {World ;}
   Bullets:Array [0..Max_Bullets] of TBullet;
   Builds:Array [0..Max_Buildings] of TBuild;
   Procedure set_Level(lev:byte);
   Procedure Render;
   Procedure Engine;
   Procedure Move;  
   procedure Put_City;
   Function Add_Bullet(Point:TVect;Ang,Speed:single;Tip:byte):integer;
   Constructor Create;
 end;

Var World:TWorld;

implementation

//Sound
{$REGION 'Sound'}
procedure TWorld.Play_Sound(Sound:integer);
begin
{}
end;
{$ENDREGION}
{$REGION 'DRAW SPRITES'}
procedure TWorld.Put_Sprite(Point:TVect;Ang,Mash,Size:Single);
begin
  glPushMatrix();
  glTranslatef(Point[0]*Mash,Point[1]*Mash,0);
  glRotatef(Ang,0,0,1);
  glBegin(GL_Quads);
  glTexCoord2f(0,1);    glVertex2f(-Size,-Size);
  glTexCoord2f(0,0);    glVertex2f(-Size, Size);
  glTexCoord2f(1,0);    glVertex2f( Size, Size);
  glTexCoord2f(1,1);    glVertex2f( Size,-Size);
  glEnd();
  glPopMatrix();
end;

procedure TWorld.Put_Sprite(Point:TVect;Mash,Size:Single);
begin
  glBegin(GL_Quads);
  glTexCoord2f(0,1);
  glVertex2f(Point[0]*Mash-Size,Point[1]*Mash-size);
  glTexCoord2f(0,0);
  glVertex2f(Point[0]*Mash-Size,Point[1]*Mash+size);
  glTexCoord2f(1,0);
  glVertex2f(Point[0]*Mash+Size,Point[1]*Mash+size);
  glTexCoord2f(1,1);
  glVertex2f(Point[0]*Mash+Size,Point[1]*Mash-size);
  glEnd();
end;

procedure TWorld.Put_Tile(Point:TVect;Mash,Size:Single;a,b:byte);
begin
  glBegin(GL_Quads);
  glTexCoord2f((a  )*0.25,(b+1)*0.25);glVertex2f((Point[0]-Size)*Mash,(Point[1]-Size)*Mash);
  glTexCoord2f((a  )*0.25,(b  )*0.25);glVertex2f((Point[0]-Size)*Mash,(Point[1]+Size)*Mash);
  glTexCoord2f((a+1)*0.25,(b  )*0.25);glVertex2f((Point[0]+Size)*Mash,(Point[1]+Size)*Mash);
  glTexCoord2f((a+1)*0.25,(b+1)*0.25);glVertex2f((Point[0]+Size)*Mash,(Point[1]-Size)*Mash);
  glend;
end;

procedure TWorld.Put_Tile_4(Point:TVect;Ang,Mash,Size:Single;a,b:byte);
begin
  glPushMatrix();
  glTranslatef(Point[0]*Mash,Point[1]*Mash,0);
  glRotatef(Ang,0,0,1);
  glBegin(GL_Quads);               
  glTexCoord2f((a+1)*0.25,(b+1)*0.25);
glVertex2f(-Size,-Size);
  glTexCoord2f((a  )*0.25,(b+1)*0.25);
glVertex2f(-Size, Size);
  glTexCoord2f((a  )*0.25,(b  )*0.25);
glVertex2f( Size, Size);
  glTexCoord2f((a+1)*0.25,(b  )*0.25);
glVertex2f( Size,-Size);
  glEnd();
  glPopMatrix();
end;

procedure TWorld.Put_Bullet(B:TBullet);
Var Size:single;
begin
  Size:=1;
   case B.Tip of
     BT_None: ;
     BT_Rocket:Put_Tile_4(minus(B.Point,multi(K,0.5)),B.Ang,Mash,0.1*Mash,0,0);
     BT_Sputnick:Put_Tile_4(minus(B.Point,multi(K,0.5)),B.Ang,Mash,0.4*Mash,1,0);
     BT_Bullet:Put_Tile_4(minus(B.Point,multi(K,0.5)),B.Ang,Mash,0.2*Mash,3,0) ;
     BT_Anti_Rocket:Put_Tile_4(minus(B.Point,multi(K,0.5)),B.Ang,Mash,0.1*Mash,2,0) ;
     BT_Bot:Put_Tile_4(minus(B.Point,multi(K,0.5)),180,Mash,0.1*Mash,0,1);
   end;
end;
{$ENDREGION}

{$REGION 'Stars & sity'}
Procedure TWorld.Draw_StarField;
Var SFK1,SFK2,SFS1,SFS2:Single;
begin
 SFK1:=0.1;
 SFK2:=0.11;
{ SFK1:=0.05;
 SFK2:=0.0551;}
 SFS1:=2.5;
 SFS2:=2.5;
 glEnable(Gl_Texture_2d);
 glBindTexture(GL_Texture_2d,Self.SF1_Tex);
 glColor3f(5,5,5);
  glBegin(GL_Quads);
  glTexCoord2f(K[0]*SFK1,K[1]*SFK1);          glVertex2f(-8,-8);
  glTexCoord2f(K[0]*SFK1,K[1]*SFK1+SFS1);     glVertex2f(-8,+8);
  glTexCoord2f(K[0]*SFK1+SFS1,K[1]*SFK1+SFS1);glVertex2f(+8,+8);
  glTexCoord2f(K[0]*SFK1+SFS1,K[1]*SFK1);     glVertex2f(+8,-8);
  glEnd();
 glBindTexture(GL_Texture_2d,Self.SF2_Tex);
 glColor3f(5,5,5);
  glBegin(GL_Quads);
  glTexCoord2f(K[0]*SFK2,K[1]*SFK2);          glVertex2f(-8,-8);
  glTexCoord2f(K[0]*SFK2,K[1]*SFK2+SFS2);     glVertex2f(-8,+8);
  glTexCoord2f(K[0]*SFK2+SFS2,K[1]*SFK2+SFS2);glVertex2f(+8,+8);
  glTexCoord2f(K[0]*SFK2+SFS2,K[1]*SFK2);     glVertex2f(+8,-8);
  glEnd();   
end;

procedure TWorld.Put_City;
Var i,j:byte;C:Single;
begin
  glDisable(GL_Texture_2d);
  glBegin(GL_Quads);
  glColor3f(0.5,0.5,1);
  glVertex2f(+koef[0]/2,koef[1]/2+(-koef[1]/5)-K[1]/1.5);
  glVertex2f(-koef[0]/2,koef[1]/2+(-koef[1]/5)-K[1]/1.5);
  glColor3f(0,0,0);
  glVertex2f(-koef[0]/2,koef[1]/2);
  glVertex2f(+koef[0]/2,koef[1]/2);
  glEnd();
  glenable(GL_Texture_2d);
  glBindTexture(GL_TExture_2d,City_Tex);
  for j := 2 to 6 do
  for i:=0 to 5 do
    begin
      c:=6*City[i].life/5;
      glColor3f(C,C,C);
      Put_Tile(plus(City[i].Poz,Vector((j mod 2)/6-K[0]/j,-j/6-K[1]/j))
        ,1,City[i].Size,city[i].tip,city[i].tip+j);
    end;
end;
{$ENDREGION}
//RENDER
{$REGION 'RENDER'}
Procedure TWorld.Render_Scene;
Var i{,j}:integer;
begin
 glPushMatrix();
 Draw_StarField;{stars}
 {drav city}
 Put_City;
 if Length(Scene)>trunc(tact/50) then
 Menu.put_Text( Scene[trunc(tact/50)+1],3,4);
 Menu.put_Text(IntToStr(Self.tact),3,3);
 {Bullets}
 glColor3f(5,5,5);
 glBindTexture(GL_Texture_2d,Bullets_Tex);
 for i:= 0 to Max_Bullets do
  if (Bullets[i].tip<>BT_None) then
   begin
    Put_Bullet(Bullets[i]);
   end;
 glPopMatrix();
 {}
end;

Procedure TWorld.Render_Menu;
begin
 Draw_StarField;{stars}
 Put_City;
 Menu.Render;
end;

Procedure TWorld.Render;
begin
 if not finded then Find_Coord;
 case self.Game_State of
  gs_none:;
  gs_action:Self.Render_Scene;
  gs_menu:Self.Render_Menu;
  gs_help:;
  gs_intro:;
 end;
end;
{$ENDREGION}
//Move
{$REGION 'Move'}
Procedure TWorld.Move;
Var
i,j:integer;A:Single;
Gravity,S:TVector;
begin
 if (Self.Game_State<>gs_action) then exit; 
 if Stress>0 then 
  Stress:=Stress-1;
 Gravity:=Vector(0,-0.00003);
 {bullets}
 for i:=0 to Max_Bullets do
  if (Bullets[i].tip<>BT_None) then
  begin
    case Bullets[i].Tip of
     BT_None: ;
     BT_Rocket:
      begin
       Bullets[i].Vect:=plus(Bullets[i].Vect,Gravity);
       if (Bullets[i].Point[1]<koef[1]/2+0.5) then
        begin
          boom(Bullets[i].Point);
          Stress:=100;
          Bullets[i].Tip:=BT_None;
          for j:=0 to 5 do
            if abs(City[j].Poz[0]-Bullets[i].Point[0])<City[j].Size then
             if City[j].life>0 then City[j].life:=City[j].life-1;
        end;
      end;
     BT_Sputnick:
      begin
        if Random(100)=0 then
          add_Bullet(Bullets[i].Point,220+Random(100){Random(360)},0.007,1);
      end;
     BT_Bullet:Bullets[i].Vect:=plus(Bullets[i].Vect,Gravity);
     BT_Anti_Rocket:begin
      s:=minus(Bullets[i].Point,Bullets[i].Cel);
      if (abs(s[0])<=Bullets[i].Speed) and
       (abs(s[1])<=Bullets[i].Speed) then
       begin
        Poom(Bullets[i].Point);
        Bullets[i].Tip:=BT_None;
        for j:= 0 to Max_Bullets do
         if (Bullets[j].tip<>BT_None) then
          if (Get_S(Bullets[i].Point,Bullets[j].Point)<0.5) then
            begin
             Bullets[j].Tip:=BT_None;
             Poom(Bullets[j].Point);
            end;
       end;
     end;
     BT_Bot:
      begin
       Bullets[i].Vect:=plus(Bullets[i].Vect,Gravity);
      end;
    end;
    Bullets[i].Point:=plus(Bullets[i].Point,Bullets[i].Vect);
    Bullets[i].Act:=Bullets[i].act+1;
    if (Bullets[i].act>5) then
    begin
     add_Particle(Bullets[i].Point[0],Bullets[i].Point[1],0,0,0,0,7,80);
     Bullets[i].act:=0;
    end;
     if (Bullets[i].Point[0]>koef[0]/2+1) or
       (Bullets[i].Point[0]<-koef[0]/2-1) or
       (Bullets[i].Point[1]>-koef[1]/2+1) or
       (Bullets[i].Point[1]<koef[1]/2-1) then
        Bullets[i].Tip:=BT_None;
    { begin
      Bullets[i].Tip:=BT_None;
     end; {bullet drop on planet}
  end;
end;
{$ENDREGION}
//engine
{$REGION 'Engine'}
procedure TWorld.Engine;
var i,a:integer;
begin
 tact:=tact+1;
 if tact/50=(tact div 50) then
  if Length(Scene)>trunc(tact/50) then
  case Scene[trunc(tact/50)+1] of
   ' ':;
   'a','A':add_Bullet(Vector(-koef[0]/2+Random(1000)/1000*(koef[0]),-koef[1]/2+0.1),220+Random(100),0.007,1);
   'b','B':
    begin
     add_Bullet(Vector(-koef[0]/2+Random(1000)/1000*(koef[0]),-koef[1]/2+0.1),220+Random(100),0.007,1);
     add_Bullet(Vector(-koef[0]/2+Random(1000)/1000*(koef[0]),-koef[1]/2+0.1),220+Random(100),0.007,1);
     add_Bullet(Vector(-koef[0]/2+Random(1000)/1000*(koef[0]),-koef[1]/2+0.1),220+Random(100),0.007,1);
    end;
   'c','C':add_Bullet(Vector(koef[0]/2+Random(5)*0.1+0.1,-koef[1]/2-0.5),180,0.025,2);
   'd','D':
    begin
     add_Bullet(Vector(-koef[0]/2+Random(1000)/1000*(koef[0]),-koef[1]/2+0.1),220+Random(100),0.007,5);
     add_Bullet(Vector(-koef[0]/2+Random(1000)/1000*(koef[0]),-koef[1]/2+0.1),220+Random(100),0.007,5);
     add_Bullet(Vector(-koef[0]/2+Random(1000)/1000*(koef[0]),-koef[1]/2+0.1),220+Random(100),0.007,5);
    end;
  end;

// addition of random rocket in the sky
{ if Random(80)=0 then
  add_Bullet(Vector(-koef[0]/2+Random(1000)/1000*(koef[0]),-koef[1]/2+0.1),220+Random(100),0.007,1);
 if Random(1000)=0 then
  add_Bullet(Vector(koef[0]/2+Random(5)*0.1+0.1,-koef[1]/2-0.5),180,0.025,2);
//boom
//faike - out fire
 if Random(400)=0 then
 begin
  Out_Fire:=40;
  out_city:=Random(6);
 end;

 if Out_Fire>0 then Out_Fire:=Out_Fire-1;
 if (Out_Fire>0) and (Out_Fire mod 6 = 0) then
   add_Bullet(City[out_city].Poz,90+(Random(10)+5)*((out_city mod 2)*2-1),0.03,3);
}

end;

procedure TWorld.Poom(P:TVect);
var i,j:integer;
begin
for i:=0 to 10 do
  begin
   j:=Random(360);
   add_Particle(P[0],P[1],0,
            (Random(100)-50)*0.00005,(Random(100)-50)*0.00005,0,2+Random(2),50+Random(31));
  end;
end;

procedure TWorld.Boom(P:TVect);
var i:integer;
begin
 for i:=0 to 18 do
  begin
   add_Particle(p[0],p[1],0,Get_X_V(0+i*10,0.005+Random(3)*0.001),Get_Y_V(0+i*10,0.006)+0.014,0,Random(2),80+Random(25));
  end;
 for i:=0 to 40 do
  begin
   add_Particle(p[0],p[1],0,(Random(100)-50)*0.00004,i/40*0.02,0,4+Random(3),80+Random(20));
   add_Particle(p[0],p[1],0,(Random(100)-50)*0.00004,i/40*0.02,0,4+Random(3),80+Random(20));
  end;
end;
{$ENDREGION}

{$REGION 'Set level'}
Procedure TWorld.set_Level;
begin
 case Lev of
  0:Self.Scene:=' a   b  aa     aa    aa   aabaa    c  baab     aa  c   a  c       aaabbbbb   bc      c     ddd  a   b  aa     aa    aa   aabaa    c  baab     aa  c   a  c       aaabbbbb   bc';
  1:;
  2:;
 end;
end;
 
{$ENDREGION}
//add bullet
{$Region 'add someth'}
Function TWorld.Add_Bullet(Point:TVect;Ang,Speed:single;Tip:byte):integer;
Var i:integer;
begin
 for i:= 0 to Max_Bullets do
  if (Bullets[i].tip=BT_None) then
   begin
    Bullets[i].tip:=T_Bullet_Tip(Tip);
    Bullets[i].Point:=Point;
    Bullets[i].Ang:=Ang;
    Bullets[i].Vect:=Get_Vect(Ang,Speed);
    Bullets[i].Speed:=Speed;
    {
    case Bullets[i].tip of
     BT_Rocket:Bullets[i].Speed:=0.04;{usual Rocket}
    { BT_Bullet:Bullets[i].Speed:=0.03;{UnUsual:) Rocket}
    {end;}
    Add_Bullet:=i;
    break;
   end;
end;

{$EndRegion}

{$REGION 'Find Coords'}
procedure TWorld.Find_Coord;
Var i:byte;Size:single;a,b:TVect;
begin
  a[0]:=-koef[0]/2;
  b[0]:=koef[0]/2;
  a[1]:=-koef[1]/2;
  b[1]:=koef[1]/2;
  Size:=(b[0]-a[0])/7;
  for i:=0 to 5 do
    begin
      City[i].Size:=Size/2;
      City[i].Poz[0]:=a[0]+(b[0]-a[0])/7*(i+1);
      City[i].Poz[1]:=a[0]+Size*2;
      City[i].life:=3;
    end;
  finded:=true;
end;
{$ENDREGION}

Constructor TWorld.Create;
Var i:byte;
begin
  Mash:=1;
  K:=Vector(0,0);
  finded:=false;
 // Game_State:=gs_action;
  Game_State:=gs_Menu;
  set_Level(0);
end;

end.
