unit frmLevel;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, ImgList, uFieldData, ActnList, Buttons,
  ComCtrls, ToolWin, XMLDoc, XMLIntf;

type
  TFormLevel = class;

  TSaveLevelEvent = procedure(Sender:TFormLevel) of object;

  TFormLevel = class(TForm)
    EditorField: TScrollBox;
    EditorPaintBox: TPaintBox;
    Panel1: TPanel;
    edFieldName: TEdit;
    Icons: TImageList;
    ActionList1: TActionList;
    acSave: TAction;
    ImageList1: TImageList;
    ToolBar1: TToolBar;
    ToolButton1: TToolButton;
    procedure EditorPaintBoxMouseDown(Sender: TObject;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure EditorPaintBoxMouseMove(Sender: TObject; Shift: TShiftState;
      X, Y: Integer);
    procedure EditorPaintBoxMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure EditorPaintBoxPaint(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure edFieldNameChange(Sender: TObject);
    procedure acSaveExecute(Sender: TObject);
  private
    { Private declarations }
    fLevelIndex: Integer;
    fXMLNode: IXMLNode;
    fLastFilename: String;
    fFieldData: TFieldData;
    fMouseDown: Boolean;
    fMouseDrag: Boolean;
    fOnSaveLevel: TSaveLevelEvent;
    procedure paintField();
    procedure newField(sx, sy, lim: Integer; name:String);
    procedure setNewFileName(val:String);
    procedure setData(data:String);
    function getData:String;
    procedure setLimit(lim:Integer);
    function getLimit:Integer;
    property lastFileName:String read fLastFilename write setNewFileName;
  public
    { Public declarations }
    procedure saveCurrent;
    procedure NewLevel(AIndex, AWidth, AHeight, ALimit: Integer;
      AName:String; xml:IXMLNode);
    procedure makeWallFrame;
    function GetLevelInfo:String;
    property data:String read getData write setData;
    property limit:Integer read getLimit write setLimit;
    property levelIndex: Integer read fLevelIndex write fLevelIndex;
    property xmlNode:IXMLNode read fXMLNode write fXMLNode;
    property OnSaveLevel: TSaveLevelEvent read fOnSaveLevel write fOnSaveLevel;
  end;

var
  FormLevel: TFormLevel;

implementation

uses frmMain;

{$R *.dfm}

procedure TFormLevel.EditorPaintBoxMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbLeft then
  begin
    fMouseDown := true;
  end else
  if Button = mbMiddle then
  begin
    fMouseDrag := true;
  end;
  EditorPaintBoxMouseMove(Sender, Shift, X, Y);
end;

procedure TFormLevel.EditorPaintBoxMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
  if Assigned(fFieldData) then
  begin
    if fMouseDown and
      (fFieldData.cells[x div CELL_SIZE, y div CELL_SIZE] <> FormMain.fSelectedType) then
    begin
      fFieldData.cells[x div CELL_SIZE, y div CELL_SIZE] := FormMain.fSelectedType;
      paintField;
    end;
    // -
    if fMouseDrag then
    begin
    
    end;
    FormMain.StatusBar.Panels[0].Text := 'x:y ' +
      IntToStr(x div CELL_SIZE) + ':' +
      IntToStr(y div CELL_SIZE);
    FormMain.Statusbar.Panels[1].Text := 'Limit: ' +
      IntToStr(fFieldData.limit);
  end;
end;

procedure TFormLevel.EditorPaintBoxMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbLeft then
  begin
    fMouseDown := false;
  end;
  if Button = mbMiddle then
  begin
    fMouseDrag := false;
  end;
end;

procedure TFormLevel.EditorPaintBoxPaint(Sender: TObject);
begin
  if Assigned(fFieldData) then
  begin
    paintField;
  end;
end;

procedure TFormLevel.newField(sx, sy, lim: Integer; name:String);
begin
  if Assigned(fFieldData) then
  begin
    fFieldData.Destroy;
  end;
  fFieldData := TFieldData.Create(sx, sy, lim);
  EditorField.HorzScrollBar.Range := CELL_SIZE * sx;
  EditorField.VertScrollBar.Range := CELL_SIZE * sy;
  fFieldData.name := name;
  paintField;
end;

procedure TFormLevel.paintField;
var
  i, j:Integer;
  buff:TBitmap;
begin
  if not Assigned(fFieldData) then Exit;
  try
    buff := TBitmap.Create();
    buff.Width := EditorPaintBox.Width;
    buff.Height := EditorPaintBox.Height;

    // Paint Range
    buff.Canvas.Pen.Color := clMaroon;
    buff.Canvas.Rectangle(0, 0, fFieldData.sizeX * CELL_SIZE, fFieldData.sizeY * CELL_SIZE);

    // Paint grid
    buff.Canvas.Pen.Color := clBtnShadow;
    for i := 0 to fFieldData.sizeX do
    begin
      buff.Canvas.MoveTo(i * CELL_SIZE, 0);
      buff.Canvas.LineTo(i * CELL_SIZE, fFieldData.sizeY * CELL_SIZE);
    end;
    for j := 0 to fFieldData.sizeY do
    begin
      buff.Canvas.MoveTo(0, j * CELL_SIZE);
      buff.Canvas.LineTo(fFieldData.sizeX * CELL_SIZE, j * CELL_SIZE);
    end;

    // Paint cells
    for i := 0 to fFieldData.sizeX - 1 do
    for j := 0 to fFieldData.sizeY - 1 do
    begin
      //EditorPaintBox.Canvas.Pixels[i * CELL_SIZE, j * CELL_SIZE] := clBlack;
      //EditorPaintBox.Canvas.Draw(i * CELL_SIZE, j * CELL_SIZE, );
      Icons.Draw(buff.Canvas, i * CELL_SIZE, j * CELL_SIZE,
        ord(fFieldData.cells[i, j]), dsNormal, itImage);
    end;

    EditorPaintBox.Canvas.Draw(0, 0, buff);
    //EditorPaintBox.Canvas.FillRect(Rect(10, 10, 100, 100));
  finally
    buff.Free;
  end;
end;

procedure TFormLevel.setNewFileName(val: String);
begin
  fLastFilename := val;
  //FormMain.Caption := 'Editor - ' + val;
end;

procedure TFormLevel.FormCreate(Sender: TObject);
begin
  //newField(20, 10, 10);
end;

procedure TFormLevel.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caFree;
end;

procedure TFormLevel.edFieldNameChange(Sender: TObject);
begin
  if Assigned(fFieldData) then
  begin
    fFieldData.name := edFieldName.Text;
    Caption := edFieldName.Text;
  end;
end;

procedure TFormLevel.NewLevel(AIndex, AWidth, AHeight, ALimit: Integer;
  AName:String; xml:IXMLNode);
begin
  newField(AWidth, AHeight, ALimit, AName);
  edFieldName.Text := AName;
  Caption := AName;
  levelIndex := AIndex;
  fXMLNode := xml;
end;

function TFormLevel.getData: String;
begin
  if Assigned(fFieldData) then
    Result := fFieldData.data
  else
    Result := '';
end;

procedure TFormLevel.setData(data: String);
var
  i, j, k: Integer;
begin    
  if Assigned(fFieldData) then
  begin
    k := 1;
    for i := 0 to fFieldData.sizeX - 1 do
    for j := 0 to fFieldData.sizeY - 1 do
    begin
      if k <= Length(data) then
      begin
        fFieldData.cells[i, j] := TFieldObject(ord(data[k]) - ord('a'));
        Inc(k);
      end
      else
      begin
        fFieldData.cells[i, j] := foEmpty;
      end;
    end;
  end;
end;

function TFormLevel.getLimit: Integer;
begin
  if Assigned(fFieldData) then
    Result := fFieldData.limit
  else
    Result := 0;
end;

procedure TFormLevel.setLimit(lim: Integer);
begin
  if Assigned(fFieldData) then
    fFieldData.limit := lim;
end;

procedure TFormLevel.makeWallFrame;
begin
  if Assigned(fFieldData) then
  begin
    fFieldData.makeWallFrame;
    paintField;
  end;
end;

function TFormLevel.GetLevelInfo: String;
var
  cnt, x, y: Integer;
begin
  cnt := 0;

  for x := 0 to fFieldData.sizeX - 1 do
  for y := 0 to fFieldData.sizeY - 1 do
    if fFieldData.cells[x, y] = foDiamond then
      Inc(cnt);
  Result := 'Level size: '
    + IntToStr(fFieldData.sizeX)
    + 'x'
    + IntToStr(fFieldData.sizeY)
    + chr(10)
    + 'Diamonds count: '
    + IntToStr(cnt)
    + chr(10)
    + 'Diamonds limit: '
    + IntToStr(fFieldData.limit);
end;

procedure TFormLevel.acSaveExecute(Sender: TObject);
begin
  //
  saveCurrent;
end;

procedure TFormLevel.saveCurrent;
begin
  if Assigned(fOnSaveLevel) then
  begin
    fXMLNode.Attributes['name'] := fFieldData.name;
    fXMLNode.Attributes['limit'] := fFieldData.limit;
    fXMLNode.NodeValue := fFieldData.data;
    fOnSaveLevel(self);
  end;
end;

end.
