unit uFieldData;

interface

const
  CELL_SIZE:Integer = 32;

type                     
  TFieldObject = (
    foEmpty = 0,
    foWall,
    foNettings,
    foRock,
    foDiamond,
    foBarrel,
    foKeyRed,
    foKeyGreen,
    foKeyBlue,
    foKeyMagenta,
    foDoorRed,
    foDoorGreen,
    foDoorBlue,
    foDoorMagenta,
    foPlayerStart,
    foPlayerStop,
    foMonsterLHRight,
    foMonsterLHDown,
    foMonsterLHLeft,
    foMonsterLHUp,
    foMonsterRHRight,
    foMonsterRHDown,
    foMonsterRHLeft,
    foMonsterRHUp
  );

  TArray2 = Array of Array of TFieldObject;

  TFieldData = class
    private
      fSizeX: Integer;
      fSizeY: Integer;
      fLimit: Integer;
      fName: String;
      fCells:TArray2;
      procedure setCell(x, y:Integer; val:TFieldObject);
      function getCell(x, y:Integer):TFieldObject;
      function getData():String;
    public
      constructor Create(X, Y, L:Integer);
      //
      procedure makeWallFrame;
      // 
      property cells[x, y:Integer]: TFieldObject read getCell write setCell;
      property sizeX:Integer read fSizeX;
      property sizeY:Integer read fSizeY;
      property limit:Integer read fLimit write fLimit;
      property name:String read fName write fName;
      property data:String read getData;
  end;

implementation

uses SysUtils;

{ TFieldData }

constructor TFieldData.Create(X, Y, L: Integer);
var
  i:Integer;
begin
  fSizeX := X;
  fSizeY := Y;
  fLimit := L;
  SetLength(fCells, X);
  for i := 0 to sizeX - 1 do
  begin
    SetLength(fcells[i], Y);
  end;
end;

function TFieldData.getCell(x, y: Integer): TFieldObject;
begin
  if(x >= fSizeX) or (x < 0) or
    (y >= fSizeY) or (y < 0) then
  begin
    Result := foWall;
  end
  else
  begin
    Result := fcells[x, y];
  end;
end;

function TFieldData.getData: String;
var
  x, y: Integer;
begin
  Result := '';
  for x := 0 to sizeX - 1 do
  begin
    for y := 0 to sizeY - 1 do
    begin
      Result := Result + chr(ord(cells[x, y]) + ord('a'));
    end;
    Result := Result + chr(10);
  end;
end;

procedure TFieldData.makeWallFrame;
var
  i, j: Integer;
begin
  for i := 0 to fSizeX - 1 do
  begin
    fCells[i, 0] := foWall;
    fCells[i, fSizeY - 1] := foWall;
  end;
  for i := 0 to fSizeY - 1 do
  begin
    fcells[0, i] := foWall;
    fCells[fSizeX - 1, i] := foWall;
  end;
end;

procedure TFieldData.setCell(x, y:Integer; val: TFieldObject);
begin
  if(x < fSizeX) and (x >= 0) and
    (y < fSizeY) and (y >= 0) then
  begin
    fcells[x, y] := val;
  end;
end;

end.
 