unit SD_bar;

interface
uses
  Omega, LexUtils;

type
  tBarButton = class
    X, Y, W, H, T, N : Integer;
    MD : Boolean;
    function MouseInB : Boolean;
  end;

  tBridgeButton = class(tBarButton)
    procedure Draw;
    procedure Work;
  end;

  tZoomButton = class(tBarButton)
      sX : Single;
    procedure Draw;
    procedure Work;
  end;

  tBar = class
    R1, R2 : TMRect;
    BridgeButton : array[1..4] of tBridgeButton;
    ZoomB : array[1..2] of tZoomButton;
  procedure Init;
  procedure Draw;
  procedure Work;
  end;

procedure Init;
var
  Bar : tBar;

implementation
uses
  SD_procs, SD_creator, SD_map, SD_cmap,
  SD_expl, SD_Guns, SD_game,
  SD_gproc, SD_bridges, Setup, SD_materials;

function tBarButton.MouseInB;
begin
  if  (MouseX > X)
  and (MouseY > Y)
  and (MouseX < X + W)
  and (MouseY < Y + H) then
    Result := True
  else
    Result := False;
  MD := Result;
end;

procedure tBridgeButton.Draw;
begin
  if T > 0 then
  begin
    if MD then
      Pr2D_Rect(X + 2, Y + 2, W - 4, H - 4, $707070, 150, 1);
    FragmentBridge[T].DrawMenu(X, Y, W div 5);
  end;
end;

procedure tBridgeButton.Work;
begin
  SD_creator.WiC   := 1;
  SD_creator.WiC2  := T;
  SD_bridges.UseBB := N;
  T := 0;
end;

procedure tZoomButton.Draw;
begin
  if MD then
    Pr2D_Rect(X + 1, Y + 1, W - 2, H - 2, $707070, 150, 1);
  ASprite_Draw(m_ZoomB, X, Y, W, H, T);
end;

procedure tZoomButton.Work;
var
  I : Integer;
begin
  if ( (T = 1) and (sMT < 128 ))
  or ( (T = 2) and (sMT > 16 )) then
  begin
    sMT := sMT * Sx;
    Map.Hero.X := Map.Hero.X * Sx;
    Map.Hero.Y := Map.Hero.Y * Sx;
    SD_Expl.ReSize(Sx);
    for I := 0 to Length(Rocket) - 1 do
      Rocket[I].P.X := Rocket[I].P.X * Sx;
    SetDSize;
    CollisMap.SetSize;
   end;
end;

procedure tBar.Init;
const
  Bord = 5;
var
  iX, iY : Integer;
begin
  R1.X := 0;
  R1.Y := 0;
  R1.H := Setup.WinH;
  R1.W := Setup.WinW div 8;

  R2.X := R1.X + Bord;
  R2.Y := R1.Y + Bord;
  R2.W := R1.W - 2*Bord;
  R2.H := R1.H - 2*Bord;

  for iX := 1 to 4 do
  begin
    BridgeButton[iX] := tBridgeButton.Create;
    with BridgeButton[iX] do
    begin
      W := (R2.W - 4);
      H := BridgeButton[1].W;
      X := (R1.W - W) div 2;
      Y := 50 + R2.Y + 2 + (iX - 1)*BridgeButton[1].H;
      T := random(MofFB) + 1;
      N := iX;
    end;
  end;

  for iX := 1 to 2 do
  begin
    ZoomB[iX] := tZoomButton.Create;
    with ZoomB[iX] do
    begin
      W := (R2.W - 4) div 2;
      H := ZoomB[iX].W;
      X := R1.W div 2 - (2-iX) * W;
      Y := R2.H - ZoomB[iX].H - H div 4;
      T := iX;
    end;
  end;
  ZoomB[1].Sx := 2;
  ZoomB[2].Sx := 0.5;
end;

procedure tBar.Draw;
var
  I : Integer;
begin
//___________________________
  with R1 do
    Pr2D_Rect(X, Y, W, H, $A79777, 200, 1);
  with R1 do
    Pr2D_Rect(X, Y, W, H, $FFFFFF, 200, 0);
  with R2 do
    Pr2D_Rect(X, Y, W, H, $7B7B7B, 255, 0);
//___________________________
  Text_Draw(FontA1, R2.X + 2, R2.Y + 15, PChar('  ' + Int2S(HeroL)));


  for I := 1 to 4 do
    BridgeButton[I].Draw;

  for I := 1 to 2 do
    ZoomB[I].Draw;
end;

procedure tBar.Work;
var
  I  : Integer;
  MD : Boolean;
begin
// 
  if Omega.Mouse_Press(M_LBUTTON) then
  begin
    // bridge button
    if  (UseBB > 0)
    and (BridgeButton[UseBB].T = 0) then
      BridgeButton[UseBB].T := Wic2;
    UseBB := 0;
    //____________________
    WiC   := 0;
    WiC2  := 0;
    MD := True;
  end
  else
    MD := False;
//  
  for I := 1 to 4 do
  if  (BridgeButton[I].MouseInB)
  and (BridgeButton[I].T > 0)
  and (MD) then
  begin
    BridgeButton[I].Work;
    Exit;
  end;
  for I := 1 to 2 do
  if  (ZoomB[I].MouseInB)
  and (MD) then
    ZoomB[I].Work;

end;

procedure Init;
begin
  Bar := tBar.Create;
  Bar.Init;
end;




end.
